C  *******************************************************************
C  *                                                                 *
C  *            *****     SUBROUTINE ICO_POLART     *****            *
C  *                                                                 *
C  *    INTERPOLATE TRANSFORM ONTO POLAR GRID                        *
C  *                                                                 *
C  *    REWRITTEN TO MAKE USE OF CALLED ARRAYS RATHER THAN COMMONS   *
C  *    13-JAN-87  STEVE FULLER                                      *
C  *                                                                 *
C  *    SUBROUTINE CALLED BY PROGRAM EMICOFV                         *
C  *    MODIFIED  2-DEC-88 BY T. BAKER (MINOR CLEANUP)               *
C  *    LAST MODIFIED 29-MAY-1989 BY T. BAKER                        *
C  *******************************************************************
C
      SUBROUTINE ICO_POLART (APOL, BPOL, ARGPOL, FPOL, NSAMPL, NANG,
     1      AT, BT, IRAD, MINR, MAXR)
C
      IMPLICIT NONE
C
      INTEGER*4 IRAD, NANG, MINR, MAXR, NSAMPL, NA, NR
      REAL*4    AT((2*IRAD+1)*(IRAD+2)), BT((2*IRAD+1)*(IRAD+2))
      REAL*4    APOL(NSAMPL,NANG), BPOL(NSAMPL,NANG)
      REAL*4        ARGPOL(NSAMPL,NANG), FPOL(NSAMPL,NANG)
      REAL*4    PI, RADIAN, DEGREE, ANG, DELANG
      REAL*4    RADIUS, DEL_RADIUS, COSANG, SINANG, A, B, X, Y
C  ----------------------------------------------------------------------------
C
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      ANG        = 0.0
      DELANG     = PI/(NANG-1)
      DEL_RADIUS = FLOAT(MAXR-MINR)/FLOAT(NSAMPL-1)
C
      DO NA = 1,NANG
        RADIUS = MINR
        COSANG = COS(ANG)
        SINANG = SIN(ANG)
        DO NR = 1,NSAMPL
          X = RADIUS*COSANG
          Y = RADIUS*SINANG
          CALL ICO_INTERP (X, Y, A, B, AT, BT, IRAD)
          APOL(NR,NA)   = A
          BPOL(NR,NA)   = B
          FPOL(NR,NA)   = SQRT(A*A + B*B)
          ARGPOL(NR,NA) = ATAN2(B,A)*DEGREE
          RADIUS        = RADIUS + DEL_RADIUS
        ENDDO
        ANG = ANG+DELANG
      ENDDO
C
      RETURN
      END

c###############################################################################
c
c
c      zminC
C          Cartesian version of zmin 
c            if FMIN >0 - does a least squares fit of ln(F av)
c             vs resln and then zeros the transform in resolution 
c             rings taking the average value at each resolution from 
c             the plot
c            if FMIN<0 - calculates the average F over the entire
c             resolution range and zeros the transform point 
c             if F < /FMIN/*average F
C            Unless the logical variable ZERO is set, the routine only
c             determines the resolution dependent cutoff but does not
c             zero the transform
c
c
c                        LAST MODIFIED S.Fuller 18-juli-85
c            
c.............................................................................
c
      subroutine zminc(AT,BT,IRAD,MINR,MAXR,FMIN,FCUT)
c
      implicit none
      integer*4 NSAMPL,MINR,MAXR,IRAD
      integer*2 IPRINT1, IPRINT2
      COMMON /COMPR/ IPRINT1, IPRINT2
      real*4 AT((2*IRAD+1)*(IRAD+2)),BT((2*IRAD+1)*(IRAD+2))
      real*4 FAV(300),RESLN(300)
      real*4 FMIN,FCUT(300),RAD
      real*4 A,B,SIGA,SIGB,R,FZERO,FZEROA(300),FSUM
      integer*4 NTOT,INDEX,INDEXM
      integer*4 NZERO,NZEROA(300),I,J, IR, IX, IY, N(300)
      logical ZERO/.FALSE./
c
c............................................................................
c
      FSUM = 0.0
      NTOT = 0
C
      do IR=1,2*MAXR
        FZEROA(IR)=0.0
        N(IR)=0
        FAV(IR)=0.0
        RESLN(IR)=0.0
      end do
c
      do IX=0,MAXR
        do IY=0,MAXR
          RAD=sqrt(float(IX*IX+IY*IY))
          IR=nint(RAD)
          if( IR .ge. MINR .and. IR .le. MAXR) then
            INDEX=IY*(2*IRAD+1)+(IRAD+1+IX)
            INDEXM=INDEX-2*IX
            A=sqrt(AT(INDEX)*AT(INDEX)+BT(INDEX)*BT(INDEX))
            B=sqrt(AT(INDEXM)*AT(INDEXM)+BT(INDEXM)*BT(INDEXM))
            FAV(IR)=FAV(IR)+A+B
            N(IR)=N(IR)+2
            RESLN(IR)=RESLN(IR)+2.0*RAD      
          ENDIF
        end do
      end do
c
      do IR=MINR,MAXR
        if(N(IR).gt.0 .and. FAV(IR).gt.0.0) then
          FSUM=FSUM+FAV(IR)
          NTOT=NTOT+N(IR)
          RESLN(IR)=RESLN(IR)/float(N(IR))
          FAV(IR)=alog(FAV(IR)/float(N(IR)))
        else 
          FAV(IR) =0.0
        ENDIF
      end do
c
      if(FMIN.gt.0.0) then
         NSAMPL=MAXR-MINR+1
         call linfit(RESLN(MINR), FAV(MINR), RESLN, NSAMPL, 0,
     $        A, SIGA, B, SIGB, R)
Ca        write(6,'('' linfit: A,SIGA,B,SIGB,R '',
CA      1      5(2x,g12.6))') A,SIGA,B,SIGB,R
         print *,' linfit: A,SIGA,B,SIGB,R ',
     1        A,SIGA,B,SIGB,R

        ENDIF
c
      FSUM=FSUM/float(NTOT)
      NZERO=0
C
      do IR=MINR,MAXR
        NZEROA(IR)=0        
        if(FMIN .gt. 0.0) then
          FCUT(IR)=exp(RESLN(IR)*B+A)*FMIN
        else if(FMIN .lt. 0.0) then
          FCUT(IR)=FSUM*(-1.0*FMIN)
        else 
          FCUT(IR)=FSUM            
        ENDIF
      end do
C
        if(IPRINT1.NE.1) return  ! Don't complete fraction zeroed calc if
                         !  IPRINT1 < 1
c
      do IX=0,MAXR
        do IY=0,MAXR
          RAD=sqrt(float(IX*IX+IY*IY))
          IR=nint(RAD)
          if( IR .ge. MINR .and. IR .le. MAXR) then
            INDEX=IY*(2*IRAD+1)+(IRAD+1+IX)
            INDEXM=INDEX-2*IX
            A=sqrt(AT(INDEX)*AT(INDEX)+BT(INDEX)*BT(INDEX))
            B=sqrt(AT(INDEXM)*AT(INDEXM)+BT(INDEXM)*BT(INDEXM))
            if(A.lt.FCUT(IR))then
            if(ZERO) then
              AT(INDEX)=0.0
              BT(INDEX)=0.0
            ENDIF
            NZEROA(IR)=NZEROA(IR)+1
            NZERO=NZERO+1
            ENDIF
            if(B.lt.FCUT(IR))then
            if(ZERO) then
              AT(INDEXM)=0.0
              BT(INDEXM)=0.0
            ENDIF
            NZEROA(IR)=NZEROA(IR)+1
            NZERO=NZERO+1
            ENDIF
          ENDIF
        end do
      end do
c
      do IR=MINR,MAXR
        if(N(IR).gt.0) then
          FZEROA(IR)=float(NZEROA(IR))/float(N(IR))
          FAV(IR)=exp(FAV(IR))
        else
          FZEROA(IR)=0.0
          RESLN(IR)=0.0
          FAV(IR)=0.0
        ENDIF
        WRITE(6,'('' RADIUS #'',I3,
     1      '' RESLN '', F7.2,
     1      '' F AVG '',G9.4,
     1      '' F CUTOFF '',G9.4,
     1      '' # '',I5,
     1      '' # ZEROED '',I5
     1      '' % ZEROED '',F5.1)')
     1 IR,RESLN(IR),FAV(IR),FCUT(IR),N(IR),NZEROA(IR),FZEROA(IR)*100.0
      ENDDO
c
      FZERO=float(NZERO)/float(NTOT)
      write(6,'('' Total number of points '',i7,
     1      '' Total number zeroed '',i7,
     1      '' Total fraction zeroed '', g10.4)')NTOT,NZERO,FZERO      
c
      return
      end

c###############################################################################
c
c      zmin 
c            if FMIN >0 - does a least squares fit of ln(F av)
c             vs resln and then zeros the transform in resolution 
c             rings taking the average value at each resolution from 
c             the plot
c            if FMIN<0 - calculates the average F over the entire
c             resolution range and zeros the transform point 
c             if F < /FMIN/*average F
c
c                              S.Fuller
c            last modified dec-86
c..............................................................................
c
      SUBROUTINE ZMIN (FPOL, NSAMPL, NANG, MINR, MAXR, FMIN)
C
      IMPLICIT NONE
C
      INTEGER*4 NSAMPL,NANG,MINR,MAXR, MSAMPL
      PARAMETER (MSAMPL=300)
      real FPOL(NSAMPL,NANG), FAV(MSAMPL), RESLN(MSAMPL)
      real FMIN,FCUT, A, B, SIGA, SIGB, R, FZERO, FZEROA, FSUM
      integer NZERO,NZEROA,I,J
c..............................................................................
c
      if(NSAMPL.ge.MSAMPL) then
        WRITE(6,'('' ZMIN:Dimension problem NSAMPL = '',I8)') NSAMPL
        RETURN
      ENDIF
C
      FSUM = 0.0
      DO I = 1,NSAMPL
        FAV(I)   = 0.0
        RESLN(I) = MINR+((I-1)*(MAXR-MINR))/float(NSAMPL-1)
        DO J = 1,NANG
          FAV(I) = FAV(I) + FPOL(I,J)
          FSUM   = FSUM   + FPOL(I,J)
        ENDDO
        FAV(I) = alog(FAV(I)/float(NANG))
      ENDDO
c
      if(FMIN.gt.0.0) then
        CALL LINFIT (RESLN,FAV,RESLN,NSAMPL,0,A,SIGA,B,SIGB,R)
        WRITE(6,'('' LINFIT: A,SIGA,B,SIGB,R '',5(2X,G12.6))')
     1  A,SIGA,B,SIGB,R
        WRITE(1,'('' LINFIT: A,SIGA,B,SIGB,R '',5(2X,G12.6))')
     1  A,SIGA,B,SIGB,R
        ENDIF
C
      FSUM  = FSUM/float(NSAMPL*NANG)
      NZERO = 0
C
      WRITE(1,'('' RADIUS# TF PIXEL RADIUS     F AVG     F CUTOFF
     1 # ZEROED   % ZEROED'',/,1X,66(''-''))')
      WRITE(6,'('' RADIUS# TF PIXEL RADIUS     F AVG     F CUTOFF
     1 # ZEROED   % ZEROED'',/,1X,66(''-''))')
C
      DO I = 1,NSAMPL
        NZEROA = 0        
        IF(FMIN.GT.0.0) THEN
           FCUT = exp(RESLN(I)*B+A)*FMIN
         ELSE IF (FMIN .LT. 0.0) THEN
           FCUT = FSUM*(-1.0*FMIN)
         ELSE 
           FCUT = FSUM            
        ENDIF
C
        DO J = 1,NANG
          IF(FCUT.GT.FPOL(I,J)) THEN
            FPOL(I,J) = 0.0
            NZEROA    = NZEROA + 1
            NZERO     = NZERO + 1
          ENDIF
        ENDDO
        FZEROA = FLOAT(NZEROA)/FLOAT(NANG)
        FAV(I) = EXP(FAV(I))
        WRITE(1,1000) I, RESLN(I), FAV(I), FCUT, NZEROA, FZEROA*100.0
        WRITE(6,1000) I, RESLN(I), FAV(I), FCUT, NZEROA, FZEROA*100.0
      ENDDO
C
      FZERO = FLOAT(NZERO)/FLOAT(NANG*NSAMPL)
      WRITE(6,'('' Total number zeroed '',I7,
     1      ''  Total fraction zeroed '', G10.4)') NZERO, FZERO
      WRITE(1,'('' Total number zeroed '',I7,
     1      ''  Total fraction zeroed '', G10.4)') NZERO, FZERO
      FMIN = FZERO
C
      RETURN
 1000      FORMAT(I6,F12.2,F16.2,F12.2,I7,F13.1)
      END

C  *****************************************************************************
C  *                                                             *
C  *              *****     SUBROUTINE ICO_COMLIN_A     *****                   *
C  *                                                                        *
C  *       FOR PAIRS OF COMMON LINES                                          *
C  *                                                                        *
C  *    SUBROUTINE CALLED BY SUBROUTINES RSYM, ICOFV_OMEAG1 AND ICOFV_OMEGA2   *
C  *     OF EMICOFV PROGRAM.                                                   *
C  *                                                             *
C  *      WARNING: This version OF ICO_COMLIN is not compatible with that used   *
C  *             by later segments of the program set !!!!!!!!!!             *
C  *                                                             *
C  *    Routine was modified in June 1985 by S. Fuller to flag common lines    *
C  *     which are degenerate (IFLAG(K)) and to calculate a the minimum radius *
C  *     at which the pairs of common lines will be separated by 1 TRANSFORM   *
C  *     unit (RAD(K).                                                         *
C  *                                                             *
C  *      Codes for common lines:                                             *
C  *                                                             *
C  *      IFLAG = 0 - the lines match none of these conditions.                  *
C  *                                                             *
C  *            1 - the two lines are superimposed (ang1 = ang2).             *
C  *                                                             *
C  *            2 - the two lines are identical to an earlier pair.            *
C  *                (i.e. ang1 = ang1' and ang2 = ang2'                        *
C  *                                                             *
C  *            3 - the two lines are equal to an earlier pair but in reversed *
C  *                order (i.e. ang1 = ang2' and ang2 = ang1'                   *
C  *                                                             *
C  *            4 - the two lines are the Friedel of an earlier pair             *
C  *                (i.e. ang1 = ang2' + pi and ang2 = ang1' + pi              *
C  *                   or ang1 = ang1' + pi and ang2 = ang2' + pi             *
C  *                                                             *
C  *            5 - the two lines are Friedel mates (ang1 = ang2 + pi)             *
C  *                                                             *
C  *      ***     6 - the two lines sum to pi (ang1 + ang2 = pi)                 *
C  *                                                             *
C  *      MODIFIED 16-juli-1985 by S. Fuller                                  *
C  *    MODIFIED to make the 1 and 4 flags match their definitions in the      *
C  *     comments.                                                             *
C  *       MODIFIED 15-MAR-88 BY SDF                                              *
C  *    LAST MODIFIED  5-DEC-88 BY T. BAKER (DELETED FLAG #6; FIXED FLAG #4A,  *
C  *                 (PASS VALUE OF RADIAN TROUGH SUBROUTINE ARGUMENT LIST)    *
C  *****************************************************************************
C
      SUBROUTINE ICO_COMLIN_A (ROT,THETA,PHI,ALPHA1,ALPHA2,IFLAG,RAD,
     1 RADIAN)
C
      IMPLICIT NONE
C
      REAL*4    ROT(3,3,37), RADIAN, XX, PHI, THETA
      REAL*4    ALPHA1(37), ALPHA2(37), X(3), Y(3), VEC(3), X0(2)
      REAL*4        RAD(37), TMP, DIFF, VV, TEST
      real*8 deg_to_rad
      INTEGER*2 I, J, K, IFLAG(37)
C  ----------------------------------------------------------------------------
C

      deg_to_rad = acos(-1.0d0)/180.0d0

      X(1)   = SIN(deg_to_rad*THETA)*COS(deg_to_rad*PHI)
      X(2)   = SIN(deg_to_rad*THETA)*SIN(deg_to_rad*PHI)
      X(3)   = COS(deg_to_rad*THETA)
      XX     = SQRT(X(1)*X(1) + X(2)*X(2))
      X0(1)  = -X(2)/XX
      X0(2)  =  X(1)/XX
C
      DO 200 K = 1,37
        RAD(K)   = 1000.0
        IFLAG(K) =    0
C
        DO I = 1,3
          Y(I) = 0.0
          DO J = 1,3
            Y(I) = Y(I) + ROT(I,J,K)*X(J)
          ENDDO
        ENDDO
C
        VEC(1) = X(2)*Y(3) - X(3)*Y(2)
        VEC(2) = X(3)*Y(1) - X(1)*Y(3)
        VEC(3) = X(1)*Y(2) - X(2)*Y(1)
        VV     = SQRT(VEC(1)*VEC(1) + VEC(2)*VEC(2) + VEC(3)*VEC(3))
        IF(VV.LT.0.1)  THEN
          IFLAG(K) = 1
          GO TO 200
        ENDIF
C
        DO I = 1,3
          VEC(I) = VEC(I)/VV
        ENDDO
C
        DO I = 1,3
          Y(I) = 0.0
          DO J = 1,3
            Y(I) = Y(I) + ROT(J,I,K)*VEC(J)
          ENDDO
        ENDDO
C
        TMP = X0(1)*VEC(1) + X0(2)*VEC(2)
        IF(TMP.LT.-1.0) TMP = -1.0
        IF(TMP.GT.1.0)  TMP =  1.0
        ALPHA1(K) = ACOS(TMP)/RADIAN
        TMP       = X0(1)*Y(1) + X0(2)*Y(2)
        IF(TMP.LT.-1.0) TMP = -1.0
        IF(TMP.GT.1.0)  TMP =  1.0
        ALPHA2(K) = ACOS(TMP)/RADIAN
        IF(VEC(3).LT.0.) ALPHA1(K) = 360.-ALPHA1(K)
        IF(Y(3).LT.0.)   ALPHA2(K) = 360.-ALPHA2(K)
        DIFF = ABS(ALPHA1(K)-ALPHA2(K))
        IF(DIFF.GT.180.) DIFF = 360.-DIFF
        RAD(K) = 2.0*SIN(deg_to_rad*DIFF/2.0)
        IF(RAD(K).GT.1.0e-6) then
           RAD(K) = AMAX1(1.0E-8,1.0/RAD(K))
         ELSE
           RAD(K) = 1000.0
        ENDIF
C         ******************************************
C---------*  IFLAG = 1: TEST IF ALPHA1 = ALPHA2    *
C         ******************************************
        IF(ALPHA1(K).EQ.ALPHA2(K)) THEN
          IFLAG(K) = 1
          RAD(K)   = 1000.0
       ENDIF
  200       CONTINUE
C
C  ************************************************************************
C  *    CHECK FOR IDENTICAL PAIRS OF COMMON LINES                         *
C  *                                                                      *
C  *    IFLAG = 2: THE TWO LINES ARE IDENTICAL TO ANOTHER PAIR.           *
C  *               ALPHA1(K) = ALHPA1(J) AND ALPHA2(K) = ALPHA2(J)        *
C  *                                                                      *
C  *    IFLAG = 3: THE TWO LINES ARE IDENTICAL TO ANOTHER PAIR, BUT IN    *
C  *               REVERSED ORDER                                         *
C  *               ALPHA1(K) = ALHPA2(J) AND ALPHA2(K) = ALPHA1(J)        *
C  ************************************************************************
      DO K = 1,36
        IF(IFLAG(K).EQ.0) THEN
          DO J = K+1,37
            IF(IFLAG(J).EQ.0) THEN
              TEST = ABS(ALPHA1(K)-ALPHA1(J))
              IF(TEST.LE.0.1) THEN
                TEST = ABS(ALPHA2(K)-ALPHA2(J))
                IF(TEST.LE.0.1) THEN
                  IFLAG(J) = 2
                ENDIF
              ENDIF
            ENDIF
          ENDDO
        ENDIF
      ENDDO
      DO K = 1,36
        IF(IFLAG(K).EQ.0) THEN
          DO J = K+1,37
            IF(IFLAG(J).EQ.0) THEN
              TEST = ABS(ALPHA1(K)-ALPHA2(J))
              IF(TEST.LE.0.1) THEN
                TEST = ABS(ALPHA2(K)-ALPHA1(J))
                IF(TEST.LE.0.1) THEN
                  IFLAG(J) = 3
                ENDIF
              ENDIF
            ENDIF
          ENDDO
        ENDIF
      ENDDO
C  ***************************************************************************
C  *    CHECK FOR ONE PAIR BEING THE FRIEDEL MATE OF ANOTHER PAIR            *
C  *                                                                         *
C  *    IFLAG = 4: THE TWO LINES ARE THE FRIEDEL OF ANOTHER PAIR.            *
C  *               ALPHA1(K) = ALPHA1(J) + PI AND ALPHA2(K) = ALPHA2(J) + PI *
C  *            OR ALPHA1(K) = ALPHA2(J) + PI AND ALPHA2(K) = ALPHA1(J) + PI *
C  ***************************************************************************
      DO K = 1,36
        IF(IFLAG(K).EQ.0) THEN
          DO J = K+1,37
            IF(IFLAG(J).EQ.0) THEN
              TEST = ABS( ABS(ALPHA1(K)-ALPHA1(J))-180.0)
            IF(TEST.LE.0.1) THEN
              TEST = ABS( ABS(ALPHA2(K)-ALPHA2(J))-180.0)
              IF(TEST.LE.0.1) IFLAG(J) = 4
              ENDIF
            TEST = ABS( ABS(ALPHA1(K)-ALPHA2(J))-180.0)
            IF(TEST.LE.0.1) THEN
              TEST = ABS( ABS(ALPHA2(K)-ALPHA1(J))-180.0)
              IF(TEST.LE.0.1) IFLAG(J) = 4
              ENDIF
            ENDIF
          ENDDO
        ENDIF
      ENDDO
C
      DO K = 1,37
        IF(IFLAG(K).EQ.0) THEN
C           ****************************************************
C-----------*    TEST FOR TWO LINES THAT ARE FRIDEL RELATED    *
C           *    IFLAG = 5: ABS(ALPHA1(K)-ALPHA2(K))=180       *
C           ****************************************************
          TEST = ABS( ABS(ALPHA1(K)-ALPHA2(K))-180.0 )
          IF(TEST.LE.0.1) IFLAG(K) = 5
C           ****************************************************
C-----------*    TEST FOR TWO LINES THAT ADD TO 180 DEGREES    *
C           *    IFLAG = 6: ABS(ALPHA1(K)+ALPHA2(K))=180       *
C           ****************************************************
C  ***************************************************************
C  *    I DON'T BELIEVE THIS IS AN ALLOWED TEST: TSB 5/DEC/88    *
C  ***************************************************************
C          TEST = ABS( ABS(ALPHA1(K)+ALPHA2(K))-180.0 )
C          IF(TEST.LE.0.1) IFLAG(K) = 6
        ENDIF
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C! PCHISQ: calling routine for prob chisqr from chiprob
c
c
C      Returns values for prob of REDUCED chisqr.
c
c      When prob chisqr .lt. 1e-8 for NFREE odd use the average of
c         the even chisqr's on either side. This happens for chisqr
c         greater than 35.
c
c            sdf 3-nov-87
c................................................................
      subroutine pchisq(RED_CHISQR,NFREE,PROB,IPROB)
C
      implicit none
      real*4      RED_CHISQR,CHISQR,PROB,IPROB, chiprob
      integer      NFREE
C............................................................
      CHISQR=RED_CHISQR*NFREE
      if(mod(NFREE,2).eq.0 .or. CHISQR.lt.35 .or. NFREE.le.2) then
          PROB = chiprob(CHISQR,NFREE)
      else
          PROB = (chiprob(CHISQR,NFREE-1)+chiprob(CHISQR,NFREE+1))/2.0
      ENDIF
      IPROB = 1.0 - PROB
      return
      end

C  ****************************************************************************
C ! CHIPROB: returns the probability that chisqr exceeds X for NFREE 
c      degrees of freedom
c
c            The routine uses a recursion formula
c
c            P(chisq>x,nfree) = P(chisqr>X,nfree-2) +
c
c                                     (nfree-1)/2
c                              (chisqr/2)          * exp(-x/2)
c                              _____________________________
c                                    gamma(nfree/2)
c
c            and the facts that
c
c                  P(chisqr>x,2) = exp(-x/2)
c
c                  P(chisqr>x,1) = 2*normal(-sqrt(x))
c
c            to set up a iterative calculation.
c
c            The procedure calls the real*8 function normal
c
c            Routine was translated from the Alogol 60 routine in
c              CACM algorithm 299.
c
c                  --- sdf 2-nov-87
c.....................................................................
c
c
      function chiprob(X_VAL,NFREE)
C
      implicit none
      real*4 X_VAL, chiprob
      real*8 Z, C, E, A, Y, S, X
      logical EVEN, BIGX
      integer NFREE
      real*8 NORMAL

C............................................................................

      if(X_VAL.lt.0.0 .or. NFREE.lt.1) then
          write(6,*) ' chiprob: values of X and NFREE out of bounds ' 
          write(6,*) ' X = ', X_VAL, ' NFREE = ', NFREE
          CHIPROB = 0.0
          return
      ENDIF
      if(X_VAL.eq.0.0) then
          CHIPROB = 1.0
          return
      ENDIF

      X = 1.0d0 * X_VAL

      A = 0.5d0 * X

      EVEN = (mod(NFREE,2).eq.0)

      BIGX = (A.gt.(25*0.693147180) )      
c-----------------      limit for accurate val of exp(-a)

      if( (EVEN .or. NFREE.gt.2) .and. .not.BIGX) then
          Y = exp(-A)
      ENDIF
      if(EVEN) then
          S = Y
      else
          S = 2.0d0*normal(-sqrt(X))
          if (S.eq.0.0d0 .and. NFREE.gt.2 ) then
             S = 2.d0*exp(-A)
          ENDIF
c---------------      use the even function value as estimate when the 
c--------------                  normal routine returns 0                  
      ENDIF

      if (NFREE.gt.2) then
          X = 0.5d0*(NFREE-1.0)
          if (EVEN) then
             Z = 1.0d0
          else
            Z = 0.5d0
          ENDIF
          if (BIGX) then
            if(EVEN) then
                E = 0.0
            else 
                E = 0.572364942925  
c--------------------             value of ln(sqrt(pi))
            ENDIF
            C = log(A)
            do while (Z.le.(X*1.0001) )
                E = log(Z)+E
                S = exp(C*Z-A-E) + S
                Z = Z + 1.0d0
            end do
            CHIPROB = S
          else
            if(EVEN) then
                E = 1.0d0
            else
                E = 0.564189583548/sqrt(A) 
c---------------------                  value of  1/sqrt(pi)
            ENDIF
            C = 0.0d0
            do while (Z.le.(X*1.0001) )
                E = E*A/Z
                C = C+E
                Z = Z + 1.0d0
            end do
            CHIPROB = C*Y+S
          ENDIF
      else
          CHIPROB = S
      ENDIF
      return

      end

C! NORMAL: real*8 calculation of integral of normal distribution
C
C      Calculates the value of:
c
c                          x
c                        /
c            (1/sqrt(2*pi)) (  exp(-u**2/2)du
c                        )
c                         /
c                         -oo
c
c            by means of a series expansion developed by A.M. Murray 
c             of Aberdeen Univ. The routine is translated from the 
c             Algol 60 of CACM algorithm # 209 and is preferred for
c             speed. The routine is accurate to 7 decimal places but
c             sets values above 6.0 to 1.0 and below -6.0 to 0.0.
c
c
c                  sdf 2-nov-87
c
c...................................................................


      function NORMAL(X)
      implicit none
      real*8 X, NORMAL, Z, Y, W      
      if(X.eq.0.0) then
          Z=0.0
      else
          Y=abs(X)/2.0
      if(Y.ge.3.0) then
            Z=1.0
      else if(Y.lt.1.0) then
            W = Y*Y
            Z = ((((((((0.000124818987 * W
     1            -0.001075204047) * W + 0.005198775019 ) * W
     2            -0.019198292004) * W + 0.059054035642 ) * W
     3            -0.151968751364) * W + 0.319152932694 ) * W
     4            -0.531923007300) * W + 0.797884560593 ) * Y * 2.0
      else
            Y = Y-2.0
            Z = (((((((((((((-0.000045255659 * Y
     1      +0.000152529290) * Y -0.000019538132) * Y
     2      -0.000676904986) * Y +0.001390604284) * Y
     3      -0.000794620820) * Y -0.002034254874) * Y
     4      +0.006549791214) * Y -0.010557625006) * Y
     5      +0.011630447319) * Y -0.009279453341) * Y
     6      +0.005353579108) * Y -0.002141268741) * Y
     7      +0.000535310849) * Y +0.999936657524
         ENDIF
      ENDIF
      if(X.gt.0.0) then
         NORMAL = (Z+1.0)/2.0
      else
         NORMAL = (1.0-Z)/2.0
      ENDIF
      return
      end

c##############################################################################
c
c      subroutine linfit
c
c      least squares fit to straight line
c
c
c      X       -array of points for indep variable
c      Y       -array of points for dep variable
c      SIGMAY  -array of standard deviations for Y data points
c      NPTS      -number of pairs of dat points
c      MODE      -determines method of weighting least squares
c            +1 (instrumental) WEIGHT(I)=1./SIGMAY(I)**2
c             0 (no weighting) WEIGHT(I)=1.0
c            -1 (statistical)  WEIGHT(I)=1.0/Y(I)
c      A      -y intercept of fitted straight line
c      SIGMAA      -slope of fitted straight line
c      B      -slope of fitted straight line
c      SIGMAB      -standard deviation of B
c      R      -linear correlation coefficient
c
c
c                              S.Fuller
c
c............................................................................
c
      subroutine linfit(X,Y,SIGMAY,NPTS,MODE,A,SIGMAA,B,SIGMAB,R)
c
      implicit none
      integer*2 NPTS,MODE,I,NWGT
      real*4      X(NPTS),Y(NPTS),SIGMAY(NPTS),SIGMAA,SIGMAB,A,B,R
      real*4 SUM,SUMX,SUMY,SUMX2,SUMXY,SUMY2
      real*4 WEIGHT,DELTA,VARNCE,AVGWGT
      logical WGTSUM 
c
c.............................................................................
c
      WGTSUM=.false.
      AVGWGT=0.0
      NWGT=0
      SUM=0.0
      SUMX=0.0
      SUMY=0.0
      SUMX2=0.0
      SUMXY=0.0
      SUMY2=0.0
c
C      if MODE is +1 (SIMGAY weighting) or -1 (Y weighting)
c           , find average value of weight so that zeros
c            in the array can be filled
c
      if (MODE .eq. +1) then
        do I=1,NPTS
          if (SIGMAY(I) .eq. 0.0) then
            WGTSUM=.true.
          else 
            AVGWGT=AVGWGT+1.0/(SIGMAY(I)*SIGMAY(I))
            NWGT=NWGT+1
          ENDIF
        end do
c
c      if the SIGMAY array contains 0's replace them with the average 
c        value of SIGMAY
c
        if (WGTSUM) then
          if(NWGT .gt. 0) then
            AVGWGT=AVGWGT/NWGT
          else
            AVGWGT=1.0
          ENDIF
        ENDIF
      ENDIF
c
c
      if (MODE .le. -1) then
        do I=1,NPTS
          if (Y(I) .eq. 0.0) then
            WGTSUM=.true.
          else 
            AVGWGT=AVGWGT+1.0/abs(Y(I))
            NWGT=NWGT+1
          ENDIF
        end do
c
c      if the Y array contains 0's replace them with the average 
c        value of weight
c
        if (WGTSUM) then
          if(NWGT .gt. 0) then
            AVGWGT=AVGWGT/NWGT
          else
            AVGWGT=1.0
          ENDIF
        ENDIF
      ENDIF
c
      do I=1,NPTS
        if( MODE .le. -1) then
          if ( Y(I) .ne. 0.0) then
            WEIGHT=1.0/abs(Y(I))
          else 
            WEIGHT=AVGWGT
          ENDIF
        else if (MODE .ge. +1) then
          if ( SIGMAY(I) .ne. 0.0) then
            WEIGHT=1.0/(SIGMAY(I)*SIGMAY(I))
          else 
            WEIGHT=AVGWGT
          ENDIF
        else
          WEIGHT=1.0
        ENDIF
c
        SUM = SUM + WEIGHT
        SUMX =  SUMX + WEIGHT*X(I)      
        SUMY =  SUMY + WEIGHT*Y(I)      
        SUMX2 =  SUMX2 + WEIGHT*X(I)*X(I)      
        SUMXY =  SUMXY + WEIGHT*X(I)*Y(I)      
        SUMY2 =  SUMY2 + WEIGHT*Y(I)*Y(I)
      end do
c
c
c      Calculate coefficients and standard deviations
c
c
      DELTA = SUM*SUMX2 - SUMX*SUMX
      A = ( SUMX2*SUMY - SUMX*SUMXY)/DELTA
      B = ( SUMXY*SUM - SUMX*SUMY )/DELTA
      if ( MODE .eq. 0) then
        VARNCE = (SUMY2 + A*A*SUM + B*B*SUMX2 -2.*
     1         (A*SUMY + B*SUMXY - A*B*SUMX))/float(NPTS-2)
      else
        VARNCE = 1.0      
      ENDIF
c
      SIGMAA = sqrt(VARNCE*SUMX2 / DELTA)
      SIGMAB = sqrt(VARNCE*SUM / DELTA)
      R = (SUM*SUMXY - SUMX*SUMY)/
     1        sqrt( DELTA*(SUM*SUMY2 - SUMY*SUMY))
c
      return
      end
