/* File robem.h */
/*******************************************************************
 * ROBEM - Electron Micrograph IMaGing software.
 *
 * Include file for robem.c that includes global definitions
 * MACROS and forward declarations for robem.
 *
 * Rob Ashmore
 * Biology Department
 * Purdue University
 * Februrary 1996
 *
 *******************************************************************/

/* Struct for rectangle boxes */
typedef struct __rect_t__
  {/* RECT_T */
    int drawable;
    int x;
    int y;
    int lowx;
    int lowy;
    int width;
    int height;
  }/* RECT_T */ rect_t;

/* boxing hist */
typedef struct __bhist_t__
  {/* bhist_T */
    float rangeVal;
    int hits;
  }/* bhist_T */ bhist_t;

/* CTF struct */
typedef struct __ctf_t__
  {/* CTF_T */
    int latX0;
    int latY0;
    int latX1;
    int latY1;
    float distA;
    float distAsq;
    float distB;
    float distBsq;
    float omega;
    int pickedPts;
  }/* CTF_T */ ctf_t;

/* RGB struct */
typedef struct __rgb_t__
  {/* RGB_T */
    int red;
    int green;
    int blue;
  }/* RGB_T */ rgb_t;

/* Our colormap struct */
typedef struct __map_t__
  {/* MAP_T */
    int x;
    int y;
    int color;
    int particle_number;
  }/* MAP_T */ map_t;

/* Our lattice struct */
typedef struct __lattice_t__
  {/* LATTICE_T */
    int x;
    int y;
    int measuredX;
    int measuredY;
  }/* LATTICE_T */ lattice_t;

/* Our fullscreen struct */
typedef struct __full_t__
  {/* FULL_T */
    int dispx;
    int dispy;
    unsigned short *fullImage;
  }/* FULL_T */ full_t;

/* Our circAvg struct */
typedef struct __circAvg_t__
  {/* CIRCAVG_T */
    char  *mapName;
    float *rint;
    float *stDev;
    int   nterp;
  }/* CIRCAVG_T */ circAvg_t;

/* Our box struct */
typedef struct __box_t__
  {/* BOX_T */
    int  x;
    int  y;
    float *editBox_buffer;
    float *edit_mirror_buffer;
    int  boxZoom;
    int  inMemory;
    int  reference;
    int  nochange;
    float correlation;
    float avgdensity;
    float stdev;
  }/* BOX_T */ box_t;

typedef unsigned long	X_PIXEL;

/* Struct for Purdue INT*2 file header */
typedef struct _purdueHead_t
  {/* PURDUEHEAD_T */
    short numCols,numRows,numSecs;
    float img_bkg;
    int   nimg,nrec_last;
    int   bimg_sizes;
    float img_min,img_max;
    float map_min,map_max;
    float img_avg,img_std;
    int   file_id1, file_id2;
    short junk;
  }/* PURDUEHEAD_T */ purdueHead_t;

/* UNblob struct */
typedef struct __blobStructure_t__
  {/* blobStructure_t */
    int id;
    int size;
    float    minInt;
    float    maxInt;
  }/* blobStructure_t */ blobStructure_t;


/* *************************** *
 *       Structs for TIFF      *
 * *************************** */
typedef struct _tiffHead_t
  {/* TIFFHEAD_T */
    short   id;
    short   version;
    int     ifdOffset;
  }/* TIFFHEAD_T */ tiffHead_t;

typedef struct _tiffTag_t
  {/* TIFFTAG_T */
    short   tagID;
    short   dataType;
    int     dataCount;
    int     dataOffset;
  }/* TIFFTAG_T */ tiffTag_t;

typedef struct _tiffIFD_t
  {/* TIFFIFD_T */
    short     numDirEntries;
    tiffTag_t tagList[1];
    int       nextIFDoffset;
  }/* TIFFIFD_T */ tiffIFD_t;

typedef struct _drawCircAvg_t
  {/* DRAWCIRCAVG_T */
    char      fName[128];
    GC        colorGC;
    int       index;
    int       pixelColor;
  }/* DRAWCIRCAVG_T */ drawCircAvg_t;



/* ************************************** *
 * RAW Xlib definitions                   *
 * ************************************** */
Arg             args[256];
int             argcnt;
int             countIt;
Arg arglst[32];
X_PIXEL		black_pixel;
unsigned long             *pixels_return;
unsigned long             *dummy_pixels_return;
Display 		* dpy = (Display *) NULL;
Visual		* visual = NULL;
int		visual_depth = 0;
Window		root_window = (Window) 0;
Colormap		color_map = (Colormap) 0;
int             maxCmaps;
unsigned long   *plane_masks_return;
unsigned long   *dummy_plane_masks_return;

int             defDisplayBufferSize;  /* Volatile DEF_DISPLAY_BUFFER_SIZE */
int 		display_width;
int		display_height;

int 		buffer_width;
int		buffer_height;

int		image_width;		/* in pixels */
int		image_height;
int		file_width;		/* in pixels */
int		file_height;
int             orig_file_width;        /* Original file width */
int		image_colors;
GC		color_gc = (GC) 0;      /* Graphics contexts */
GC		rotflt_gc = (GC) 0;     
GC		fft_gc = (GC) 0;     
GC		fftContour_gc = (GC) 0;     
GC		emcoravg_gc = (GC) 0;     
GC		circAvg_gc = (GC) 0;     
GC		test_color_gc = (GC) 0;
GC		hist_color_gc = (GC) 0;
GC		editBox_gc = (GC) 0;
GC		icosa_gc = (GC) 0;     
GC		image_gc = (GC) 0;      
GC		image_gcLAT = (GC) 0;
GC		image_gcWHITE = (GC) 0;
GC		image_gcDOT = (GC) 0;
GC		image_gcDOTgreen = (GC) 0;
GC		image_gcDOTred = (GC) 0;
GC		image_gcGREEN = (GC) 0;
GC		image_gcYELLOW = (GC) 0;
GC		image_gcRED = (GC) 0;
GC		image_rotfltgcRED = (GC) 0;
GC		image_gcBLUE = (GC) 0;
GC		image_gcBLACK = (GC) 0;
GC		image_gcPLACEHOLDER = (GC) 0;
GC              image_gcGREY[258];
GC              boxing_GC;
char		        * display_name = NULL;
int		        x= 0;
int		        y= 0;
unsigned long	        gc_mask;
XGCValues		gc_values;
XColor	                * color_defs= NULL;
XColor	                * mirror_color_defs= NULL;
XColor			* gen_color_defs = NULL;
float                   pixelColor;
XEvent                  event;

int		image_xmin;             /* Image dimensions */
int		image_ymin;
int		image_xmax;
int		image_ymax;

XFontStruct *fontstruct; 	/* Font descriptor */

XFontStruct *robemFont; 	/* Global robem Font descriptor */
char thisFont [128];
XmFontList robemFontList;


/*****************
 * IMAGE BUFFERS *
 *****************/
float *float_image_buffer1;              /* For Map sections */
float *map_image_buffer;                 /* For depth cued MAP */
unsigned short *map3D_display_buffer;
unsigned short *int_image_buffer1;       /* For passing to FORTRAN routines */
int nsecs;
float *rotMap, *tmpRotMap, *this_float_image_buffer1;
float *subtract_image_buffer1, *diffSection_buffer, *subtract_map_image_buffer;
float *subtract_image_buffer2, *subtract_map_image_buffer2;
float *subMapYmxb_buffer;
float *fftImage_gold;
float *fftImage_buffer2;
float *fftImage_fourAver;  /* Special buffer for fourAver */
float *paintMap;

float  *packImg[MAXBOX];
BYTE deletePackedImage[MAXBOX];
float maxPackPixel,minPackPixel;
float scalePackFactor;
float scaleBoxFactor;

float  *inCore_buffer;

float *fix2DinCore_buffer;
float *fix2DpackImg_buffer[MAXBOX];
float *fix2DinCore_diff;
float *fix2DpackImg_diff[MAXBOX];

full_t fullScreen_buffer;

unsigned short tempBox_buffer[(MAXFILESIZE*2)] ;
float tempBox_byte_buffer[(MAXFILESIZE*2)];
float  tempBox_int_buffer[(MAXFILESIZE*2)];
float *editBox_mirror_buffer;
float *editBox_undo_buffer;

float *emcoravg_image_buffer1;
float *emcoravg_image_buffer_disk;  /* Save what was read from disk */
float *emcoravg_image_buffer_avg;  /* Save the average */

float *imgconv_image_buffer1;
float *imgconv_image_buffer_disk;  /* Save what was read from disk */
float *imgconv_image_buffer_avg;  /* Save the average */


BYTE *mirror_image_buffer1;

/********************/
/* XWindows buffers */
/********************/
XImage *image_image1;
Window image_window1;
BYTE *image_buffer1;
BYTE *image_buffer1_disp;

BYTE *emcoravg_buffer1;
XImage *emcoravg_image1;
Window emcoravg_window1;

BYTE *circAvg_buffer1;
XImage *circAvg_image1;
Window circAvg_window1;

BYTE *rotflt_buffer1;
XImage *rotflt_image1;
Window rotflt_window1;

BYTE *fft_gold_buffer1;
BYTE *fft_buffer1,*fft_buffer2,*fft_display_buffer1;
XImage *fft_image1;
Window fft_window1;
BYTE *fft_contour_buffer;
BYTE *fft_contour_buffer2;
XImage *fftContour_image1;
Window fftContour_window1;

BYTE *color_buffer1;
XImage *color_image1;
Window color_window1;

BYTE *test_color_buffer1;
XImage *test_color_image1;
Window test_color_window1;

BYTE *hist_color_buffer1;
XImage *hist_color_image1;
Window hist_color_window1;

BYTE *editBox_display_buffer2;
BYTE *editBox_display_buffer;
float  *editBox_int_buffer;
XImage *editBox_image1;
Window editBox_window1;

BYTE *icosa_buffer1;
XImage *icosa_image1;
Window icosa_window1;

int *globAccum;      /* For autoboxing cluster display */
int compress_factor = 1;
int firstAMMSTDEVtime = TRUE;
int showBoxingSwitch,showBoxingLevel;  /* For boxing histogram */
float threshLo,threshHi;

int histogram[512];
int histoEqualize;    /* Histogram Equalize or not??  */

int movieSpeed;       /* How many loop through the sleep do they want? */
int flickSpeed;       /* How many loop through the sleep do they want? */

int rotateAxis;


/* Global display and applications context pointers*/
Display         *dsply; 		/* Display variable */
XtAppContext    app_context;		/* application context */

/* All of our widgets */
static Widget toplevel_widget,          /* Root widget ID of our */
                                        /* application. */
  main_window_widget;                   /* Root widget ID of main */
                                        /* MRM fetch */

static Widget  
  interpolateWidget,
  binningWidget,
  modelWidget,
  addAverageWidget,
  fourierFiltWidget,
  circleDispWidget,
  fftMotionFeedbackWidget,
  checkBkgWidget,
  normalScreen,
  normalHelpW,
  normalExitW,
  normalMeanW,
  normalStdW,
  normalizeWidget,
  normalizeBkgWidget,
  subMagStartW,
  subMagStepW,
  subMagNumStepW,
  subMagMinRadW,
  subMagMaxRadW,
  subMagRadStepW,
  linGradWidget,
  drawingWidget,
  fftDrawingWidget,
  latPointsForm,               /* Form for defining Miller indices for lat */
  latTogglesForm,              /* Toggles for drawing FFT lattices */
  latResetW,                   /* Reset form for lattice params */
  latResetLabelW,              /* Reset label for lattice params */
  latResetButW,                /* Reset button ON lattice params Form */
  filterControlsForm,          /* COntrols for filtering lattices */
  fftImagesForm,               /* Cycle through images form */
  fftZoomToggleW,              /* Zoom the fft or not */
  ctfTogglesForm,              /* Toggle controls for CTF estimation */
  lophipForm,                  /* Form for lop/hip filter */
  displayFFTForm,              /* Display controls for FFT */
  ctfCurveForm,                /* CTF curve selector */
  ctfCurveTextW,               /* Holds CTF curve number */
  ctfScrollsForm,              /* AngMajor, focusMin and focusMaj scrolls */
  ctfFocusMinvalueW,           /* The scroll and value widgets      */
  ctfAngMajorValueW,           /*                                   */
  ctffocusMajValueW,           /*                                   */
  ctfFocusMinscrollW,          /*                                   */
  ctfAngMajorScrollW,          /*                                   */
  ctffocusMajScrollW,          /*                                   */
  ctfParamsW,                  /* CTF parameters form: */
  ctfScanStepW,                /*   parameter step size */
  ctfPixSizeW,                 /*             pixel size */
  ctfKvW,                      /*             kV  */
  ctfAmpFactorW,               /*   parameter amplitude factor */
  ctfWaveW,                    /*   parameter wavelength */
  ctfSpAberW,                  /*   parameter spherical aberration */
  ctfMajorDefocusW,            /*   parameter major defocus */
  ctfMinorDefocusW,            /*   parameter minor defocus */
  ctfWienerW,                  /*   parameter wiener */
  ctfAngMajorW,                /*   parameter angMajor */
  ctfTFactorW,                 /*   parameter temp factor */
  ctfModeW,                    /*   parameter mode  */
  ctfMagW,                     /*   parameter magnification  */
  ctfHelpLabelW,               /*   help label widget  */
  ctfHelpLabel2W,              /*   help label widget  */
  ctfMeasureLabelW,            /* Feedback for CTF measurements */
  ctfMeasureLabel2W,           /* Feedback for CTF measurements */
  movieContourScrollWidget,    /* Contour for movie frames */
  movieDeltaScrollWidget,      /* Delta Contour for movie frames */
  movieNumContScrollWidget,    /* Number of Contours for movie frames */
  contourWidget,               /* Contour for FFT regions */
  fftMeasureForm,              /* Form for measurement of FFT */
  fftFeedbackForm,             /* Form for measurement feedback */
  fftFeedbackLabelW,           /* Screen feedback label widget */
  fftMeasPixW,                 /* Measurement in pixels */
  fftMeasAngW,                 /* Measurement in Angstroms */
  fftMeasurePSTextW,             /* Holds pixelSize in Angstroms */
  fftBkgScrollW,               /* Scroll widget for background value */
  fftBkgValueW,                /* Value of background */
  zoomTextW,                   /* Holds zoom factor for main screen */
  minBoxScrollWidget,          /* Boxing min and max widgets */
  maxBoxScrollWidget,          /* Boxing min and max widgets */
  boxcolorPushW,
  lopRadiusTextW,
  hipRadiusTextW,
  lopTextW,
  hipTextW,
  lopScrollW,
  hipScrollW,
  contLabelWidget,
  hardToggleW,
  softToggleW,
  pickCTFtoggleW,
  pickLatticeToggleW,
  backFFTToggleW,
  layerLinesToggleW,
  twodLinesToggleW,
  amplitudeToggleW,
  drawLatToggleW,
  drawHiLoToggleW,
  filterFFTtoggleW,
  filtSizeWidget,
  filtDescWidget,
  filtLabelWidget,
  linearFFTtoggleW,
  logFFTtoggleW,
  nonlinFFTtoggleW,
  backTtoggleW,
  refineResetLabelW,
  refineResetButtonW,
  refineButtonW,
  tryitButtonW,
  jZerotoggleW,
  xorigUpArrowW,
  xorigDownArrowW,
  yorigUpArrowW,
  yorigDownArrowW,
  xorigTextW,
  yorigTextW,
  fftStepTextW,
  phi0ScrollW,
  phi0LabelW,
  lgendLabelW,
  lgendScrollW,
  nrotLabelW,
  nannuliLabelW,
  nannuliScrollW,
  yscaleScrollW,
  logROTFLTtoggleW,
  linROTFLTtoggleW,
  rotfltWidget,
  rotfltDrawWidget,
  coravgLowLimScrollW,
  coravgLowLimValueW,
  coravgUpArrowW,
  coravgDownArrowW,
  emcoravgDrawWidget,
  emcoravgZoomTextW,
  emcoravgRadiusTextW,
  circAvgWidget,
  circAvgScrollW,
  sectionWidget,
  lockWidget,
  icosaUpWidget,                 /* Up/Down arrow for icosa size */
  icosaDownWidget,
  paintUpWidget,                 /* Up/Down arrow for paint size */
  paintDownWidget,
  boxUpWidget,                   /* Up/Down arrow for box/section number */
  boxDownWidget,
  boxInitWidget,
  boxInitLabelW,
  boxRadiusTextWidget,
  emptyShellTextWidget,
  emmaRadiusTextWidget,
  featherRadiusTextWidget,
  imgNumTextWidget,
  infoPromptTextW,
  infoAnswerscreen,
  infoListW,
  infoBox,
  titleTextW,
  fourAverExitW,
  fixExitW,
  fix2DExitW,
  truncaddExitW,
  rotateExitW,
  pointColorW,
  pointExitW,
  boxExitW,
  optionsExitW,
  fftExitW,
  circAvgExitW,
  imgconvLabelW,
  imgconvExitW,
  imgconvHelpW,
  emcoravgExitW,
  apOldW,
  apExponentialW,
  apGaussianW,
  apLinearW,
  apOrigLabel1W,
  apOrigLabel2W,
  apOrigResetW,
  crysDispResetW,
  crysDispExitW,
  cDispSymmOrigXW,
  cDispSymmOrigYW,
  cDispZoomArrowUpW,               /* For zoom on crysDisp screen */
  cDispZoomArrowDownW,
  rotfltExitW,
  rotfltHelpW,
  statExitW,
  contourExitW,
  colorExitW,
  testcolormapWidget,
  a1024FFTWidget,
  a512FFTWidget,
  a256FFTWidget,
  a128FFTWidget,
  histWidget,
  noContrastW,                /* COntrast stretching radio buttons */
  useContrastW,
  histEqualW,
  rotfltImgNumW,
  fftImgnumArrowUpW,
  fftImgnumArrowDownW,
  rotfltImgnumArrowUpW,
  rotfltImgnumArrowDownW,
  fftImgNumW,
  lat1XtextW,
  lat1YtextW,
  lat2XtextW,
  lat2YtextW,
  millerLabelW,
  pick2pointsLabelW,
  lat1LabelW,
  lat2LabelW,
  aDscaleTextW,
  map_abangTextW,
  aoverbTextW,
  npixyTextW,
  mapScrollW,
  abScrollW,
  mapValueW,
  abValueW,
  adimScrollW,
  bdimScrollW,
  gammaXtalScrollW,
  adimTextW,
  bdimTextW,
  gammaXtalTextW,
  scaleXvalueW,
  ncellxvalueW,
  ncellyvalueW,
  scaleXScrollW,
  ncellxScrollW,
  ncellyScrollW,
  editBoxZoomTextWidget,
  editBoxPaintTextWidget,
  editBoxTextWidget,
  editBoxWidget,
  editBoxButtonW,
  removeBlemishButtonW,
  fixButtonW,
  correlateButtonW,
  correlateButton2W,
  rawButtonW,
  pcxButtonW,
  paintMapButtonW,
  mapMovieButtonW,
  circAvgButtonW,
  mapShadeButtonW,
  mapShadeExitW,
  mapShadeScreenWidget,
  rulerMagTextWidget,
  rulerPixelTextWidget,
  colormapWidget,
  colormapScrollWidget,
  colormapScrollDownWidget,
  partprocExitW,                 /* For partproc screen */
  partprocscreen,
  deltaXYpartprocScreen,
  deltaXYpartprocExitW,
  deltaXYpartprocHelpW,
  deltaXpartprocW,
  deltaYpartprocW,
  deltaOmegaPPScreen,
  deltaOmegaPPExitW,
  deltaOmegaPPHelpW,
  deltaOmegaPPW,
  cavgPPScreen,
  cavgPPExitW,
  cavgPPHelpW,
  nFoldTextW,
  nFoldLabelW,
  writeCavgToggleW,
  manipPPvalueW,
  manipPPScreen,
  manipPPExitW,
  manipPPHelpW,
  icosaWidget,
  incDecW,
  rasterStepW,                 /* Raster Step text widget for fourAver */
  backSizeW,                   /* Back Transform label widget for fourAver */
  refineLabelW,
  imposeLabelW,
  moveOrigXW,                  /* Text widgets for moving phase oprigin */
  moveOrigYW,
  moveOrigXlabelW,
  moveOrigYlabelW,
  shiftOrigXW,                  /* Text widgets for moving phase oprigin */
  shiftOrigYW,
  shiftOrigXlabelW,
  shiftOrigYlabelW,
  phasXupW,
  phasXdownW,
  phasYupW,
  phasYdownW,
  fourAverScreen,
  fourAverHelpW,
  fourAverResetW,
  ppHelpW,
  ppMinTextW,
  ppMaxTextW,
  ppMinScrollW,
  ppMaxScrollW,
  ppMinMaxFormW,
  binTextW,
  binPPUpW,
  binPPDownW,
  stepTextW,
  stepPPUpW,
  stepPPDownW,
  strtXPPW,
  endXPPW,
  maxYPPW,
  idtW,
  thetatW,
  phitW,
  omegatW,
  xtW,
  deltaXTW,
  absDeltaXTW,
  ytW,
  deltaYTW,
  absDeltaYTW,
  distanceTW,
  magtW,
  pftccW,
  prjccW,
  cmpccW,
  thetaProjW,
  phiProjW,
  omegaProjW,
  mapProjW,
  projContourTextW,
  projasuTextW,
  histControlsW,
  ppinTextW,
  ppinUpW,
  ppinDownW,
  projasuW,
  paramScanFileTextW,
  paramMapFileTextW,
  paramFileTextW,
  partPointTextW,
  parmTogsW,
  parmTogsLabelW,
  parmheadW,		
  pixsizeW,			 
  unitsW,			 
  voltsW,			 
  ampfacsW,			 
  delfMajW,		 
  delfMinW,		 
  angMajW,			 
  csCoefW,			 
  batchjobscreen,                /* For batchjob screen */
  batchjobExitW,
  batchjobFileLabelW,
  batchjobFileTextW,
  batchjobTextW,
  batchHelpLabelW,
  batchModeW,                    /* The fields of the batch job */
  batchBinW,
  batchSymW,
  batchDtheW,
  batchDphi90W,
  batchIlistW,
  batchRadLoW,
  batchRadHiW,
  batchRadStepW,
  batchAnnLoW,
  batchAnnHiW,
  batchResLoW,
  batchResHiW,
  batchJCutW,
  batchSigW,
  batchMagCenW,
  batchMagStepW,
  batchMagNumW,
  batchMagNormW,
  batch3DMapW,
  batchPrjsFileW,
  batchPftsFileW,
  batchParamFilesW,
  movementscreen,                /* For movement screen */
  movementTextWidget,
  diamTextWidget,
  movementLabelWidget,
  moveExitW,
  fourierFiltScreen,             /* For fourier filtering screen */
  fourierFiltExitW,
  fourierFiltHelpW,
  fourierFiltLabelW,
  resMinTextW,
  resMaxTextW,
  cutMinTextW,
  cutMaxTextW,
  subtractMapButtonW,            /* For subtract Map */
  subtractMapScreen,
  subtractHelpW,
  subtractExitW,
  subtractFile1TextW,
  subtractFile2TextW,
  subMap2dW,
  subMap3dW,
  smMaxRadUpW,
  smMaxRadDownW,
  smMinRadUpW,
  smMinRadDownW,
  smMinLabelW,
  smMaxLabelW,
  smMinMaxResetW,
  findMagResetW,
  findMag2ResetW,
  findMag3ResetW,
  findRadResetW,
  findRadDispW,
  findMagDispW,
  subEquationW,
  flickerTypeWidget,
  computeScalingW,
  threshScrollWidget,
  threshTextWidget,
  magSecScrollWidget,
  magSecTextWidget,
  magSecResetW,
  diffSecResetW,
  diffSec2ScrollWidget,
  diffSec2TextWidget,
  diffSecScrollWidget,
  diffSecTextWidget,
  sampleRenderWidget,            /* For sampling the render */
  calcSFButtonW,                 /* For calc SF */
  RcalcSFButtonW,                /* For calc SF on RENDER screen */
  hardRotateButtonW,             /* For Hard Rotate on RENDER screen */
  rescalescreen,                 /* For rescale screen */
  rescaleExitW,
  rescaleButtonW,
  oldMinDensLabelW,
  oldMaxDensLabelW,
  newMinDensTextW,
  newMaxDensTextW,
  invertscreen,                  /* For invert screen */
  invertExitW,
  invertButtonW,
  statusTextW,                   /* For status screen */
  statusStopW,
  versionUpWidget,
  versionDownWidget,
  versionTextWidget,
  versionStopW,
  versionScrollW,
  versionExitW,
  versionToolWidget,              /* Version Tool button */
  colorToolWidget,                /* Colormap Editor Tool button */
  colorButtonW,                   /* Colormap Editor Menu button */
  editBoxScreenWidget,
  versionScreenWidget,
  statusScreenWidget,
  colorScreenWidget,
  optionScreenWidget,
  numFramesLabelW,
  frameTextWidget,
  stepValueLabelWidget,
  stepTextWidget,
  stepToggleWidget,
  stepToggle2Widget,
  movieFormW,
  radialTextWidget,
  radialLabelWidget,
  timeLeftLabelWidget,              /* Time left on 3d Rendering */
  timeLeftBoxW,                     /* Time left Boxing */
  minMapTextW,
  maxMapTextW,
  boxHelpW,
  surfDensLabelWidget,
  fractLabelWidget,
  angxLabelWidget,
  angyLabelWidget,
  surfDensToggleWidget,              /* toggles for depth-cue stepping */
  fractToggleWidget,                 /*                                */
  angxToggleWidget,                  /*                                */
  angyToggleWidget,                  /*                                */
  thetaToggleWidget,                 /*                                */
  phiToggleWidget,                   /*                                */
  omegaToggleWidget,                 /*                                */
  unblobsizeToggleW,                 /*                                */
  unblobradiusToggleW,               /*                                */
  maxRadiusToggleWidget,             /*                                */
  pixelSizeToggleWidget,             /*                                */
  maxxToggleW,                       /*                                */
  maxyToggleW,                       /*                                */
  maxzToggleW,                       /*                                */
  maxRadiusLabelToggleWidget,
  irevToggleWidget,
  axisToggleWidget,
  unblobFormW,
  cutFormW,
  surfDensScrollWidget,
  fractScrollWidget,
  angxScrollWidget,
  angyScrollWidget,
  thetaButtonW,
  phiButtonW,
  omegaButtonW,
  thetaScrollWidget,
  phiScrollWidget,
  omegaScrollWidget,
  minRadiusScrollWidget,
  maxRadiusScrollWidget,
  pixelSizeScrollWidget,
  radialShadeWidget,
  negUnivW,
  radialCSToggleW,
  radProjCSToggleW,
  radProjToggleW,
  icosRadProjToggleW,
  radProjShadeWidget,
  stereoShadeWidget,
  loHiResWidget,
  secondShadeWidget,
  radialValueWidget,
  surfDensValueWidget,
  fractValueWidget,
  angxValueWidget,
  angyValueWidget,
  thetaValueWidget,
  phiValueWidget,
  omegaValueWidget,
  minRadiusValueWidget,
  maxRadiusValueWidget,
  pixelSizeValueWidget,
  volumeValueW,
  pointLabelWidget[7],   /* On the pointscreen */
  pointscreen,           /* The pointscreen itself */
  undoWidget,
  savePaintWidget,
  imgNumButtonW,
  statTextWidget,
  refLastIterTextW,
  refTextWidget,
  refscreen,
  fftscreen,
  imgconvscreen,
  emcoravgscreen,
  crysDispscreen,
  stepCMValueW,
  truePhiW,
  calcBinW,
  calcSfsW,
  mbImmediateUpdateW,
  immediateUpdateW,
  unblobResetW,
  unblobscreen,
  unblobSizeScrollW,
  unblobRadiusScrollW,
  unblobMinScrollW,
  unblobMaxScrollW,
  unblobSizeValueW,
  unblobFillValueW,
  unblobRadiusValueW,
  unblobMinValueW,
  unblobMaxValueW,
  cutMinValueW[3],
  cutMaxValueW[3],
  perValueW[3],
  cutMinScrollW[3],
  cutMaxScrollW[3],
  minFixTextW,
  maxFixTextW,
  pageFixTextW,
  incFixTextW,
  movie_contour_screen,
  truncaddscreen,
  truncaddButtonW,
  taUpArrowW,
  taDownArrowW,
  ta2UpArrowW,
  ta2DownArrowW,
  gbsUpArrowW,
  gbsDownArrowW,
  fixGaussStdTextW,
  fixGaussStdLabelW,
  taFillW,
  taDimXW,
  taDimYW,
  taDimZW,
  taBegXW,
  taBegYW,
  taBegZW,
  taEndXW,
  taEndYW,
  taEndZW,
  taCenterXW,
  taCenterYW,
  taCenterZW,
  rotatescreen,
  rotateButtonW,
  rotateThetaW,
  rotatePhiW,
  rotateOmegaW,
  fix2DRegionsTextW,           /* begin 2d fixes */
  fix2DStdTextW,           
  fixBegXW,                    /* begin 3d fixes */
  fixBegYW,
  fixBegZW,
  fixEndXW,
  fixEndYW,
  fixEndZW,
  fixscreen,                    /* Fix values in map */
  fix2Dscreen,                  /* Fix values in 2D */
  fixitscreen,                  /* Fix a scrollbar */
  rotfltscreen,
  fftStopW,
  fftHelpW,
  minFFTwidget,
  maxFFTwidget,
  tfacFFTwidget,
  theomaxFFTwidget,
  theominFFTwidget,
  minFFTscroll,
  maxFFTscroll,
  tfacFFTscroll,
  statscreen,
  forceAvgBkgToggleW,
  circleWidthToggleW,
  showccToggleW,                   /* SHow Corr Coef for boxing */
  showDebugToggleW,                /* SHow Debug info for boxing */
  showBoxingStatsToggleW,          /* SHow Boxing stats histogram */
  boxGrid2W,
  boxRadiusArrowUpWidget,          /* For boxRadius */
  boxRadiusArrowDownWidget,
  emptyShellArrowUpWidget,         /* For empty patrticles */
  emptyShellArrowDownWidget,
  emmaRadiusArrowUpWidget,         /* For emmaRadius */
  emmaRadiusArrowDownWidget,
  featherRadiusArrowUpWidget,      /* For featherRadius */
  featherRadiusArrowDownWidget,
  imgnumArrowUpWidget,             /* For imgNum */
  imgnumArrowDownWidget,
  zoomArrowUpWidget,               /* For zoom on main screen */
  zoomArrowDownWidget,
  flickArrowUpWidget,              /* Speed arrows for flicker */
  flickArrowDownWidget,
  movieArrowUpWidget,              /* Speed arrows for movie */
  movieArrowDownWidget,
  boxinitArrowUpWidget,            /* Arrows for find box */
  boxinitArrowDownWidget,
  movieMaxWidget,
  movieMinWidget,
  lightSourceLabelWidget,
  minLabelWidget,
  maxLabelWidget,
  movieContourLabelWidget,
  movieDeltaLabelWidget,
  movieNumContLabelWidget,
  movieBreakLabelWidget,
  drawLiteButtonW,
  drawFFTLiteButtonW,
  drawFFT2LiteButtonW,
  useContWidget,
  contourTextWidget,
  autoBoxLabelW,
  refCorrLabelW,
  autoCorrLabelW,
  autoBox_widget,
  autoBox2_widget,
  autoMinMaxLabelW,
  dilateScrollW,
  dilateLabelW,
  thinningScrollW,
  thinLabelW,
  pragScrollW,
  pragLabelW,
  refCutoffScrollW,
  refCutoffLabelW,
  rockWidget,
  flashLightWidget,
  circAvgListW,
  circAvgScreenWidget,
  circAvgTextWidget,
  circAvgLineW[4],
  circAvgLabelW[4],
  adrHiResW,
  adrDefocusW,
  adrChisqW,
  adrabResetW,
  adrb0W,
  adrb1W,
  adrb2W,
  adrb3W,
  adrb4W,
  adrb5W,
  adra0W,
  adra1W,
  adra2W,
  adra3W,
  adra4W,
  adra5W,
  a2ddrWarnW,
  a2ddrscwinW,
  a2ddrfjW,
  a2ddrfnW,
  a2ddragW,
  a2ddrtfW,
  a2ddrcsqW,
  orderBkgW,
  orderSigW,
  autoDRHelpW,
  autoDRExitW,
  autoDRScreenWidget,
  auto2DDRHelpW,
  auto2DDRExitW,
  auto2DDRScreenWidget,
  helpTruncAddW,
  helpRotateW,
  helpFixW,
  helpFix2DW,
  helpToolW,
  helpExitW,
  helpTextWidget,
  helpScreenWidget, 
  icosReplRadiusLabelXW,
  icosReplRadiusLabelYW,
  icosReplRadiusLabelZW,
  modelScreenWidget,
  modelLabelW,
  modelHelpW,
  modelExitW,
  modelFillW,
  mbRotatetTextW,
  mbRotatepTextW,
  mbRotateoTextW,
  mbRotatetScrollW,
  mbRotatepScrollW,
  mbRotateoScrollW,
  mbDimensionxTextW,
  mbDimensionyTextW,
  mbDimensionzTextW,
  mbDimensionxScrollW,
  mbDimensionyScrollW,
  mbDimensionzScrollW,
  mbCenterxTextW,
  mbCenteryTextW,
  mbCenterzTextW,
  mbCenterxScrollW,
  mbCenteryScrollW,
  mbCenterzScrollW,
  binningScreenWidget,
  binningHelpW,
  binningExitW,
  binningTextW,
  interpScreenWidget,
  interpHelpW,
  interpExitW,
  outputTextW,
  origTextW,
  appodizeHelpW,
  appodizeExitW,
  appodizeScreenWidget,
  appodBoxRadW,
  appodFeaRadW,
  fftnewXtextW,
  fftnewYtextW,
  fftFeedbackLabel1W,
  fftFeedbackLabel2W,
  fftFeedbackLabel3W,
  fftFeedbackScreen,
  fftFeedbackExitW,
  dioScreen,
  dioExitW,
  dioHelpW,
  dioThreshTextW,
  dioSlideTextW,
  boxScreenWidget,
  refCircAvgW,
  refAutomateW,
  refReadfileW,
  refReadFileLabelW,
  refResetW,
  refRegridW,
  refAllW,
  darkRefToggleW,                  /* Use dark circle ref for boxing */
  mapShadeHelpW,
  mapShadeScreenWindow,
  mapShadeStopWidget,
  boxStopWidget,
  movieDoneWidget,
  movieSaveWidget;
Widget latticeScrollW[16],latticeLabel1W[16],latticeLabel2W[16];
Widget editBoxForm,playWidget,
  playFormWidget,
  paintFormWidget,
  stopWidget, infiniWidget, infFlickerWidget;
Widget vertScroll,horiScroll;
Widget colormapValueWidget[8];  /* We draw five colors */
Widget fileSel_widget;     /* Holds the file selecxtion widget */
Widget fileDia_widget;     /* Holds the file selecxtion dialog widget */

/* The UIL file hierarchy */
static MrmHierarchy s_MrmHierarchy;     /* MRM database hierarchy ID */
/* static MrmType *dummy_class; */           /* and class variable. */
static MrmType dummy_class;            /* and class variable. */

static char *db_filename_vec[1];

static int db_filename_num =
  (sizeof db_filename_vec / sizeof db_filename_vec [0]);


/* Cursor thigs */
Cursor old_cursor;
Cursor old_newXY_cursor;
Cursor cross_cursor;
Cursor bogosity_cursor;
Cursor watch_cursor;
Cursor  zoomCursor;
Cursor  lanternCursor;
XSetWindowAttributes attrs;
Pixmap                  shape_pixmap,         /* Cursor pixmaps */
                        mask_pixmap, 
                        lanternPixmap, 
                        zoomPixmap;
Pixmap resetPixmap, noresetPixmap;
Pixmap sectionPixmap, unsectionPixmap;
Pixmap colorPixmap, uncolorPixmap;
Pixmap lockedPixmap, unlockedPixmap;
Pixmap playPixmap,stopPixmap,playingPixmap,stoppedPixmap,
  moviePixmap,unmoviePixmap,infiniPixmap,noinfiniPixmap,
  rockPixmap,norockPixmap,
  flashLightPixmap,noflashLightPixmap,goPixmap,nogoPixmap,
  exitPixmap,noexitPixmap,slowPixmap,noslowPixmap,
  helpPixmap,nohelpPixmap;



/***************************************
 * Generic C language work areas       *
 ***************************************/
fString_t fileName;                 /* What file do they want? */
fString_t image_fileName;           /* What file is the image? */

int tryColors;                  /* Keep trying until you get as many entries 
				 * in the colormap as you can */

int choice;			/* Pass this to the fortran code */
int masterProcess;              /* Is this the master or a child? */
int hiColor;                    /* Do we have > 8 bit colormap? */
int fortranBuffer;              /* Have we allocated with fortran? */

char stufBuf[NUMCOLS];          /* Init the screen */
short stufIntBuf[NUMCOLS];      /* Init the input buffer */
char stufBLKBuf[NUMCOLS];          /* Init the screen */
short stufBLKIntBuf[NUMCOLS];      /* Init the input buffer */


purdueHead_t purdueHeader;
forPifHead_t pifHeader;
char preLoadStr[256];
char databaseStr[512];
char databaseBcrdStr[512];
char databaseRootStr[512];
char databaseProjectID[512];
int databaseIter;
float shadeItDensity;
int  firstTimeLoHiRes;
float zoomItFactor;
char outStr[1024];
char versionStr[1024];
char contourStr[400];            /* Holds the contours being used */

// FILE *fIn;
int numCols, numRows;
int origWidthFromFile;           /* Remember this for pixelSize */
int fileBinCols,fileBinRows,     /* Remember these so we can write_pif */
    fileBinSecs,fileBinFactor;
int fileMrcEven, fileMrcX, fileMrcY, fileMrcZ;
int iRec;
int isBoxing,autoMinMaxThreshold;
int dispThickCircles;                  /* Display thick circles? */
int showCC;                            /* Are we giving CorrCoef feedback? */
int showDebug;                         /* Are we giving Debug feedback? */
int cycleRef;                          /* Autocycle reference correlate? */
int darkCircleRef;                     /* Use a dark circle for reference? */
int refReadFile;                       /* Use disk file reference? */
int fftMode;                           /* Are we looking at an FFT? */
int backTransform;                     /* Back transform FFT?? */
int pickLattice;                       /* Are we picking Lattice points? */
int pickCTF;                           /* Are we picking CTF points? */
int layerLines;                        /* Are we picking Lines or points? */
int contour;                           /* Are we plotting the contour? */
int contourToggle;                     /* Toggle contour plotting on & off */
int filterToggle;                      /* Are we filtering the FFT? */
int drawLatToggle;                     /* Draw lattice lines or not */
int drawHiLoToggle;                    /* Draw hi/low pass filter ? */
int latValChanged;                     /* Are we changing from scrolls or 
					* mouse?? */
int autoZoom;
int lastNumBox;
int initLastNumBox;
int lastBoxX,lastBoxY;
int boxZoom;
float zoomFactor;                      /* Let's do this in float for more
					* granularity... */
float oldZoomFactor;
float oldSizeFactor;
int editBoxZoom;
int xoffset, yoffset;
int oldGlobX, oldGlobY, globX,globY;
int virXoffset,virYoffset;
int radiusImgNum,packRadius,borderWidth;
int boxRadius, emptyShellWidth, emmaRadius, featherRadius, imgNum;
int firstGridding;            /* Make a new grid first time empty boxing */
int boxRadiusDisp, emptyShellDisp, emmaRadiusDisp, 
  featherRadiusDisp, refineDisp;
int featherMode,backgroundMode,borderType;
int clusterDisp,referenceDisp;
int findBox;
int curBox,numBoxes,numPoly,numRects;
int fullScreen;                           /* They want to get back 
					   * to normal display */
int imgInCore;
int noCallbackScroll;
int boxMethod,dispMethod,histMethod,locked,bkgrndMethod;
int makeSquareBox;                        /* Force the box to be this size */ 
int readWrite,contrastStretch,fftMinMaxLock,disp1024fft;
int writeType,readType;
int fftType;
int preloadSwitch,normLinSwitch,shadeItSwitch,zoomItSwitch,doLinAlso,doFix;
int dumpNormStats, doNoise, doApplyCtf, doTrunc;
int dumpSpherAvg;
int riverSwitch,autoBoxSwitch,boxitSwitch,doInterp,doOutFile;
int isDatabase;
int reBoxSwitch;
char outFname[2048],dumpNormFile[1024],dumpSpherAvgFile[1024];
int liteSwitch,debugSwitch,debugLevel;
int colorType,rampSwitch,rotfltSwitch,crysDispSwitch,rainbowSwitch;
int unblobSwitch;
int updateCrys;                           /* Auto update or not? */
int maxRadiusSwitch;
int fileType;
int blindWriting;
int globAsk,globAskAnswer;
int gFinished;
int lastPackedNumImgs;                   /* So we can free() intelligently */
int redLevel,greenLevel,blueLevel,brightLevel;
float histScaleFactor;
float histZoomFactor;
int scrollWidth,scrollHeight;
int headOff;
int sysError;
int paintSize;
int paintMapSwitch;
int circAvgSize,caOffsetX;
float minCA,maxCA,scaleCircFactor;
int numCircAvgDraw;
drawCircAvg_t drawCircs[5];
int stepCM;                               /* How many levels to step CM */
float rulerMag,rulerPixelWidth;
float backDispChar;                    /* Background color */
float  backChar;
int largeImage;
int inited,christmasTime;                      /* Has the UIL been 
						* instantiated? */
int drawLite;                                  /* Don't draw so much on lite
						* executions */
int numExp;                                    /* Debug exposures */
int shiftFFT;
int fftMotionFeedback;                         /* Provide pointer cues? */
int scaleItSwitch;
int dispSections,aSymmetric;
int minSurf,maxSurf,firstSec,lastSec;
float minFFTvalue,maxFFTvalue;                 /* Max and Min to display */
float globMinFFTvalue,globMaxFFTvalue;         /* Max and Min globals */
int tempFactor;                                /* FFt temp factor */
float tempFactors[1024][2048];                  /* Holds calculated tFac's */
float ctfValues[1024][2048];                    /* Holds the ctf Function */
float imageMaxFFTvalue;                        /* What was the MAX for img? */
float filtSizeValue;                           /* How large the FFT filter? */
int idim1,idim2;                               /* FFT dimensions */
int icut;                                      /* Hard(0) or SOft(1) cutoff in
						* fft filter ? */
int fftDispType;                               /* AMP Phase or Intensity */
int resoX,resoY;                               /* Resolution filter */
int lowResoFilter,hiResoFilter;                
int maxOrig,minOrig;
int applyCTF;                                  /* Apply the CTF to image? */
int ctfFilter;
int changeCTFtext;                             /* Did we chaneg the text in
						* response to the scrollbar?*/
int memoryMiser;                               /* Memory algorithms used? */
double xCen,yCen,zCen;                         /* x y z centers */
int largestDim;                                /* largest dimension of x,y,z */

/* Map globals */
int  numSecs,origNumSecs,mapNumSecs;           /* Number of map sections */
float maxMapPixel,minMapPixel,scaleMapFactor;
float cuedMaxPixel,cuedMinPixel,secMaxPixel,secMinPixel;
int makingMovie,rockIt,flashLightIt;
extern float map_min,map_max;
float    r4_min;/* minimum floating point float number on vax */
float    r4_max;/* maximum floating point float number on vax */
float surfaceDensity;
cue_t cuedMap[MAXSECTIONS];
double *rwa_tmp2d_map,*rwa_tmp_map;
int  secondMapShade, immediateMapShade, radialMapShade, stereoMapShade;
int immediateResource;          /* DO they want to inhibit immediate Update */
int  saveRadialMapShade;
int  radialProjections;
float globFract;
double globAngx,globAngy;
int shadedMaps;
BYTE globTilt;
double globTheta,globPhi,globOmega,
       globMaxRadius,globMinRadius,origGlobMaxRadius;
float globPixelSize;
int rotateXYZ;                  /* Do they want to rotate in X,Y,Z or 
				 * theta,phi,omega */
float aoverb,map_abang,map_scalef;
float deltaMapABang,deltaAoverB;
int whichStep,startMovie,endMovie;
double sinx,cosx,siny,cosy;
float cenx,ceny,cenz;
int radialStepSize;
int expandStep;
int dispIcosa;
float icosaScaleF;
int irev;
int mapStartImage,                 /* Remember these so we can use later. */
    mapSectionsPerRow;
int packStartImage,                /* Remember these so we can use later. */
    packSectionsPerRow;
float radialValue[258];
int lastDeleted;                   /* What was the last packed box they wanted
				    * deleted? */
float cosang, sinang;       /* degrees of tilt */
int   icx,icy;              /* Center of image */
float contourLevel;         /* What contour for movie contour */
int totContours;            /* Holds contour values they want */
int conTours[50];
float deltaCont,breakCont;  /* Delta contours ; Break point value*/
int numCont,useCont;        /* Number of Contours ; Use in movie???*/
int revRadProj;             /* Reverse contrast on Radial Projections? */
int radProjCS;              /* COntrast Stretch radial projections? */
int radialCS;               /* COntrast Stretch Radial Cues ?  */
int icosRadProj;            /* Do icosahedral radial projections? */
int emptyBox;               /* Are they trying to box empty particles? */

/* Appodizing stuff */
int appodizing;
int writeFFTType;

/* Circular Average stuff */
int currentCircAvg,selectedCircAvg;
circAvg_t circAvg[1000];
int caInvertContrast;       /* Invert contrast for CircAvg??? */


/* Correaltion stuff */
short *corRef;
short *ref_img;

int contrastMax, contrastMin;

float sizeFactor,recipSizeFactor;   /* Corrections based on ZOOMiness */


box_t box[MAXBOX];
int   boxRefLast[MAXBOX];           /* Which boxes were used last time for
				     * reference correlation??? */
box_t poly[MAXBOX];
int   ref_box_flag;
rect_t rectBox;
map_t imgNums[MAXBOX];
int lastImg;
int minP, maxP;                     /* What are the min/max */

/* RAMP colormap stuff */
int firstBuildY,lastBuildY,buildingCmap;

/* Lattice Points for FFT */
int fftSize;                              /* What size FFT */
lattice_t  latPt[12];                      /* LAttice/CTF Points ONE & TWO */
int currentLatPt;                         /* Which one are we doing now? */
int pickedLatPts;                         /* Have they picked the first two? */
int numLatPts;                            /* Number of lattice points we have*/
lattice_t latCoords[200];                 /* Number of lattice coords */
float axL,ayL,bxL,byL;                    /* Lattice distances */
float omega;                              /* Lattice angle */
float anga,angb,ang,astar,bstar,gamstr;   /* Lattice params */
float angaSave,angbSave,angSave,
      astarSave, bstarSave, gamstrSave,omegaSave,
      axLSave,ayLSave,bxLSave,dstar,dstarSave,
      byLSave;                            /* Lattice params - Saved  */
float latScrollValue[12];                 /* Lattice params value from scroll*/
int latLineOrder;                         /* Order of latrtice line */
int averageFFT;                           /* Avergae or single fft?? */
int fftZoom;                              /* Are we zooming the FFT? */
int fftZoomFactor;                        /* FFT Zoom factor */
int fftZoomX,fftZoomY;                    /* X,y for fft zoom */
map_t fftMeasures[3];                     /* Measure x,y in FFT */
int fftBkgValue;                          /* FFT Background value for Image */

/* fourAver stuff */
float rasterStep;                         /* Raster Step size */
int backSize;                             /* How large the back transform */
float fourAverRadius;                     
float fourAverCut;                     
int refinePhaseOrigin,imposeSymmetry;
float phaseOrigX,phaseOrigY;
int deltaPhaseShift;
int writeFourAverSF;

/* CTF stuff */
int ctfMode;                              /* CTF correction in what mode? */
int numTimesApplyCTF;                     /* How many times have we done? */
int stuffBackground;                      /* Stuff bkg after CTF correct? */
int drawCTF;                              /* Draw CTF or not */
int lopHipSwitch;                         /* Do LOP/HIP or not? */
int measureFFT;                           /* Measure FFT?? */
int ctfCurveNum;                          /* Which CTF is current */
int maxCtfCurve;                          /* What's the MAX ctf curve num? */
ctf_t ctfPts[MAXCTFS];                    /* Holds CTF curve params */
float ctfFocusMinvalue,ctfAngMajorValue,  /* Parameters for refining CTF */
      ctffocusMajValue,     
      ctfMajorDefocusValue,     
      ctfMinorDefocusValue,     
      scanStep, magnification,
      pixSize, kv,
      wavelength, spAberration,
      wiener, angMajor, ctfTFactor, ctfDstar;
int ctfDivideMode;
int doingAutoDefocusRefine, adrCircAvgDraw,adrSDraw,adrSminusBDraw,
    adrCtfsqDraw,adrDDraw,adrBDraw,adrBlankDraw;
int autoDefocus1Dmethod;
float  *ca1d, *ca1d_std;
float  *ca1d_save;
float  *ca1d_cnt,*adr_s,*adr_sminusb,*adr_ctfsq,*adr_d,*adr_b;
float  ca1d_avg;
int doingAuto2DDefocusRefine;
int drawAuto2Dctf,drawAuto2Dscurve,drawAuto2Dctflines;
float *auto2DdrawBuf,*auto2DdataBuf;
float auto2DdefMajor, auto2DdefMinor, auto2Dangle;
float  *a2dr_s,*a2dr_sminusb,*a2dr_ctfsq,*a2dr_d,*a2dr_b;
int a2drSDraw,a2drSminusBDraw,a2drCtfsqDraw,a2drDDraw,a2drBDraw;
double *poly_paras,*fftasum;
int hkl_max, order_b, order_d;
map_t scurveClick;                     /* Measure x,y in FFT */
char  gottenParamFname[1024];


/* FFT measurements */
float fftAngsPerPixel;                    /* How many angstroms per pixel? */
int fftMeasLineNum;                       /* What line num to dump to file? */
int calcFFT1DcircAvg;                     /* Give feedback? */
int disp1Dca;                             /* Disp buffer?? */
int ctfEnhance;                           /* Enhance CTF display ?? */
int bumpContrast;                         /* Bump up the contrast? */

/* ROTFLT stuff */
int idim1Rotflt;                          /* Rotflt size fft */
int lg_end;                               /* Symmetry Limit */
float *rho;                               /* Holds the backtrans */
int rotfltCrosshair;                      /* Plot center crosshair? */
int rotfltResiduals;                      /* Plot residuals contour? */
float powSpec[1000];                      /* The power Spectrum */
int radiusM1;
float yFactor,xFactor;                    /* PowSpec plotting params */
int nrot,nannuli;                         /* Symmetry = # rotations 
					   * and nannuli*/
int firstRotFlt;                          /* Is this the first pass or what? */
int logROTFLTplot;                        /* Plot power spec in log? */
float fft_step, phi0;                     /* ROTFLT Params */
int  jzero;
float sump1,sump2,sump3;                  /* Azimuthal power stats */
float logCutoff;                          /* Cutoff log plot here */
int backTransROTFLT;                      /* Back transform it ??? */
int stretchRotflt;
float minRotfltScaleF;
int newStart;
float sigMax,sigMaxTot;
float *tempRotflt_buffer;
int iopt[5];                              /* Menu choices */
int newOrigin;                            /* Calc new origin ??? */
float winScale;                           /* Scale window search for origin */
float xorigSave,yorigSave;                /* Global saved areas */
int saveLogYFactor,saveLinYFactor;        /* Save the concept of yFactor for
					   * both linear and log plots */
float *sigma;                             /* Holds the residuals */


/* crysDisp stuff */
float xorig,yorig;                        /* What is refined origin ? */
int npixx, npixy, ncellx, ncelly;         /* Stuff for mapdsp_gplot_rob() */
float aDscale,gscale;
int x_size;                               /* x_size param */
float aoverbTemp,map_abangTemp;           /* Temp interface holders */
float *crys_float_image_buffer;           /* Put image in tsb format */
int  crysDisping;                         /* Are we trying to act like a
					   * crystal??? */
int  fourierAveragedCrysDisp;             /* Did we get this by Fourier
					     Averaging ??? */
int cdNumCols,cdNumRows,cdNumSecs;
int adim,bdim;
float gammaXtal;


/* UNBLOB stuff */
int minUnblob,maxUnblob;                  /* Contour range for unblob */
int unblobSize;                           /* Size of blobs to kill */
float *unblobBuffer;                      /* Holds unblobbed map */
int radiusUnblob;                         /* Unblob radius */
int cleanUnblob;                          /* Save the largest and kill rest? */

/* CUT stuff */
int cutMinValues[3],cutMaxValues[3];      /* Cut the MAP in x,y,z dimensions*/
float perMinValues[3],perMaxValues[3];    /* For percentage cut */
int reverseSections;                      /* Reverse Sections? */
int negUniverseCut;                       /* Cut out negative Universe */
int whichIcos;                            /* Which icos display */
int destroyIcos;


/* FIXIT stuff */
Widget fixitWidget;                       /* Which widget ? */
int maxFixValue,minFixValue,pageFixValue,
    incFixValue;                          /* New params for scroll bar */
int drawFix2Dgrid,drawFix2Dstats;
int flickerFix2D;
float fix2Davg[MAXREGIONS][MAXREGIONS];
float fix2Dstd[MAXREGIONS][MAXREGIONS];

/* FIX values stuff */
Widget fixTextWidget;                     /* Which widget ? */
Widget fixMinWidget,fixMaxWidget;         /* MIn Max widgets */
int fixMode;                              /* What method of fixing? */
int fixIcosFill;

/* TruncAdd stuff */
int taArrowX,taArrowY,taArrowZ;           /* Should arrows affect these ? */


/* MRC type image file stuff */
float floatScaleFactor;
float recipFloatScaleFactor;
float int4scaleF;
char title[512];
float junkFloat;
int htype,pifMode,nimg,nrec_last,bimg_sizes,asked;
int rec_length, nrecords, nrows_per_record;
int nrecords_per_row, numRecs, recLength, totLength;
int mrc_nx, mrc_ny, mrc_nz, mrc_mode;
int mrc_nxstart, mrc_nystart, mrc_nzstart;
int mrc_mx, mrc_my, mrc_mz, mrc_mapc, mrc_mapr, mrc_maps;
int mrc_ispg, mrc_nsymbt;
int mrc_extra[29], mrc_nlabl, mrc_label[20][10], mrc_rec;
int    mrc_xlength, mrc_ylength, mrc_zlength;
float  mrc_xlengthF, mrc_ylengthF, mrc_zlengthF;
int    mrc_alpha, mrc_beta, mrc_gamma;
int    mrc_min, mrc_max, mrc_mean, mrc_std, mrc_xorigin, mrc_yorigin;
float dela, delb, delc;
float dthe, dphi_90;
int binFactor;
int t_matrix[6];

/* VOLUME stuff */
float volumeFraction[1024],
      volumeCutoff[1024];
int   volumeBin[1024];
int   volumeDisp;
int   nbins_lim,nbins;

/* AUTOBOX stuff */
int nThins;                                /* Number of thinning layers */
int nDilates;                              /* Number of dilation layers */
float prag;                                /* Cross point factor */
float refCutoff;                           /* Cutoff to throw out ref 
					    * correlations */
int refCircAvg;                            /* Circavg the ref? */
int refAutomate;                           /* Automate the ref? */
int boxitSize,boxitBoxSize,boxitFeatherSize;
char boxitFname[512];
int autoBoxSize,autoBoxBoxSize,autoBoxFeatherSize;
int autoBoxThin, autoBoxDilates, autoBoxIter;
char refReadFileName[512];

/* Pointer stuff */
int pointColor;
float apOrigXtot,apOrigYtot;


/* PARAM FILE stuff */
int dispCenters;                           /* Should we disp param centers? */
int haveParamFile,projASU;
float pixSizeP,unitsP,volts,ampFacs,delf_maj,delf_min,ang_maj,cs_coeff;
int totParms;
int icoDispMode;
int whichParam;                  /* Which file param do they want to view? */
int symmetry;                    /* What symmetry do they want to run under? */
float phiMax, theta1, theta2;
int maxPhi;                      /* Max phi in projASU */
int nview;
float *thetas,*phis;
int drawIcos;                    /* Draw the icosahedron?? */
int findNearest;                 /* Find nearest particle??? */
int whichViewMove;               /* Disp movement or view info */
int useOmega;                    /* Use omega when drawing icosahedron? */
int incSort;
int 	idtoggle=0,
        thetatoggle=0,
	phitoggle=0,
	omegatoggle=0,
	xtoggle=0,
	ytoggle=0,
        deltaXtoggle=0,
        deltaYtoggle=0,
        absDeltaXtoggle=0,
        absDeltaYtoggle=0,
        distancetoggle=0,
	magtoggle=0,
	cc1toggle=0,
	cc2toggle=0,
	cc3toggle=0;
float srange=0.0,      /* starting value for the range */
      snumb=0.0,       /* starting value for the number of particles */
      erange=512.0,    /* ending value for the range */
      enumb=512.0;     /* ending value for the number of particles */
float        startbox,endbox;    /* Start and End of the range */
int          maxYbox;            /* Max # of particles to disp in Y */
float 	     step; 		 /* Value for the step of the range */
int 	     yscale;		 /* A height scaling variable */
float        avgBadCount;        /* How many HIST values were bad? */
float 	     linelength,pixelstep;
float        ico_toolsScrollValue;    /* Value of the scrollbar */
int icosaMotioning;    /* Are we looking at motion events? */
char inParamFilename[256];
char outParamFilename[256];
char paramBoxesFilename[256];
char paramMapFilename[256];
char paramScanFilename[256];
char paramBatchJobFilename[256];
char paramBoxCoordsFilename[256];
char *batchListStr[MAXPACKEDFILES];
int totParamFiles;
int newXY;             /* Are they trying to locate a new x,y center? */
P_INFO *foundPart;     /* The one they found */
int icoSortLevel;      /* Shows progress of sorting */
int mapProj,mapProjContour;    /* Are we doing map Projections or not? */
float map_alp, map_bet, map_gam;
int mapCols, mapRows, mapSecs;
int scanCols,scanRows;
int projASUdeltaAng;
float allMax;
float deltaStdX,deltaStdY;
float minXYplot,maxXYplot;
int doManipPP,doCavgPP;
int writeCavg,cavgPPcenter;
float *cavgPPrint[MAXBOX];
int totCavgPPrint;

/* CTf fit 1d and 2d */
float *allPackImg;  
int   ctf1dCalcIt;
float *ctf1dSaveFftAvg;
int   ctf2dCalcIt;
float *ctf2dSaveFftasum,*ctf2dSaveFftasumsq;



/*****************************************************
 * emmap3dt/emsf3dbt stuff 
 *****************************************************/
strucFact_t *sFacs;
strucFact_t *fa_sFacs;
forPifHead_t faHeader;
float vol,vol_star;
float a_cell,b_cell,c_cell;
int   nsf, origNsf;
int   calcLowResMap;
int   lowRes;
float *lowRes_buffer1;
int   lowResCols,lowResRows,lowResSecs;
float pixelSizeFactor,cutFactor;
int   calcSFs,calcBin,truePhi, forceByte,forceAvgBkg,minMaxColor;
int   forceAvgBkgSwitch;
float pixelSizeSave, cutMin0Save, cutMax0Save, cutMin1Save,
      cutMax1Save, cutMin2Save, cutMax2Save;


/* Subtract 3D Map stuff */
int subtractingMap,subMap2d3d;
char subtractFilename[256];
int sub2binCol,sub2binRow,sub2binSec;
float *mags,*coefs,*cntr1,*cntr2;
int   magchk,saveMagchk,numRad;
int   subCanDrawWindow;                 /* Suppress drawing at times */
int   submapFindRad;                    /* Draw radius or mag??? */
int   rdensit, wrdensit, ccoefit, wccoefit,s_wccoefit,flickerIt;
float *rdens,*wrdens,*ccoef,*wccoef,*s_wccoef;
float radstep,diffSecScrollValue,diffSec2ScrollValue;
float magSecScrollValue,threshScrollValue;
float radScaleM,radScaleB,saveBmag;
int   thickStepFR,thinStepFR;
char  RDoutStr[128],WRDoutStr[128],CCoutStr[128],
      WCCoutStr[128],S_WCCoutStr[128];
int   drawingFindRad,diffSection,slidingWindow;
float saveMinMapPixel,saveMaxMapPixel;
int   saveSecondMap,showRadii,threshDoit,threshAbs;
int   icosaQuarter,findRadHist_toggle,findMagHist_toggle;
int   normalExcise;

/* Imgconvstuff */
int   readImgconv;
char  imgconvFilename[256];
int   imgconvNcol,imgconvNrow;
float imgconvBkg;


/* Emcoravgstuff */
int readEmcoravg;
char emcoravgFilename[256];
int emcoravgNcol,emcoravgNrow;
float emcoravgBkg;
int coravgRadius;
int lastPlotClickX,lastPlotClickY;
float coravgLowLimValue;
int useAvgCorAvg,showLowLim;


/* Normalize stuff */
int kindOfNormalize;
float normLinMean,normLinStd;

/* Partproc stuff */
int readPackedFile;

/* Fix stuff */
float batchFixStd;

/* ApplyCTF stuff */
int applyCtfFilter;
float 
  applyCtfWiener,
  applyCtfTFactor,
  applyCtfAngMajor,
  applyCtfPixelSize,
  applyCtfDefocus_in,
  applyCtfDefocus_minor_in,
  applyCtfAmpContrast_in,
  applyCtfSphericalAbbr;
int  applyCtfMode,
  applyCtfVolts;  /* int in volts NOT kiloVolts*/

/* Noise stuff */
float noiseStd;
int noiseType;

/* Trunc stuff */
float truncVal;
int truncBegX,truncEndX,truncBegY,truncEndY,truncBegZ,truncEndZ;

/* Binning stuff */
int doBinning;

/* Interp stuff */
float doInterpOrigSize,doInterpOutputSize;

/* Model stuff */
int modelType;
float *model_undo_buffer;
float modelFillValue;
int mbImmediate;
int isIcosRepl;


XmString globalTcs;


/********************************************************/
/*  FORWARD DECLARATIONS                                */
/********************************************************/
void createDefaultColormap();
void create24Colormap();
void createColormap();
void em_sleep(int i);
void ie3tvax(int *inFloat);
void vaxtie3(unsigned int *inFloat);
void calcStats(float *inbuffer, float *img_bkg, float *img_min,
	       float *img_max, float *img_avg, float *img_std,
	       int width, int height, int boxWidth);
void calcCircleStats(float *inbuffer, 
		     float *img_bkg, float *img_min,
		     float *img_max, float *img_avg, float *img_std,
		     int width, int height, int boxWidth);
/* 
 * Defined in libPIF now...
void convertBackFloat(int thisInt, float *thisFloat);
void convertFloat(int *thisInt, float thisFloat);
*/

extern int interp2d();
extern double deg2rad(double degrees);
extern double rad2deg(double rads);
double rwalog2(float thisSizeFactor);
void calcFFT(int doFFT, int thisBox, int tryBackground);
void calcNewLat();

void RwaActivateWidget();
void checkAutomatic();
void write_pif();

void clearMap(float *thisMap,int nCols,int nRows, int nSecs);

void mainRobem_sub_UNIX(unsigned short **int_buffer, 
			fString_t *fName,
			int *i, int *j);
extern void emmap_rob_shd1 (cue_t *dphshd_map, 
			    double *tmp2d, double *tmp,
			    int ncol, int nrow,
			    int sec1, int sec2, 
			    float fract, 
			    int useForMinMax, 
			    float globAngx, float stepVal,
			    float globAngy, int whichStep,
			    float passMin, int angx, int angy, int thisFract);
extern void map_1d_rob (float *map, int ncol, int nrow, int nsec, 
			float xcen, float ycen, float zcen,
			int interp, int nterp, float max_r, 
			float *rint, int *irnum, float *sum_rint,
			float *sumsq_rint, float *stdev);
extern void img_1d_rob(float *img,
		       int ncol, int nrow,
		       float xcen, float ycen,
		       int interp, int nterp,
		       float *rint, int *irnum,
		       float img_bkg);

extern void map_1d_pack_rob (BYTE *map, int ncol, int nrow, int nsec, 
			     float xcen, float ycen, float zcen,
			     int interp, int nterp, float max_r, 
			     float *rint, int *irnum, float *sum_rint,
			     float *sumsq_rint, float *stdev);
extern void rob_map_roty (float *map1, float *map2, int ncol, int nrow, 
			  int firstSec, int lastSec,
			  double rangl, float globMaxRadius, 
			  float globMinRadius);
extern void rob_map_rotx (float *map1, float *map2, int ncol, int nrow, 
			  int firstSec, int lastSec,
			  double rangl, float globMaxRadius,
			  float globMinRadius);
extern void rob_map_norot (float *map1, float *map2, int ncol, int nrow, 
			   int firstSec, int lastSec,
			   float maxRadius, float minRadius,
			   double rangl);
extern void dph_rescale_rob (int ncol, int nrow, int num_views, 
			     cue_t *cuedMap, int tryColors,
			     int *radialStepSize, int hiColor);
extern void img_imbed_ccf_rob ();
extern void img_find_peak_rob();
extern void find_phimax();
extern void get_t1t2p1();
extern void getnview_rwa();
extern void map_vol_rwa();
extern void calc_tp_rwa();
extern void calcMapStats();
extern void map2d_stats();
extern void fpoly_ctf(double  x, double  a[], double  *y, double  dyda[], 
		      int na, int order_poly1,
		      int order_poly2, double  wavelength, int idim_half, 
		      double  A, double  Cs,
		      double  pixel_size );
extern void mrqmin();
extern void free_dvector();
extern void free_ivector();
extern void free_dmatrix();
extern double *dvector();
extern float **matrix();
extern int *ivector();
extern double **dmatrix();

void unitCell_background();
void fftMeasureDraw(int button);
void findLatPt(int x, int y);
void calcDefocus(float *majorValue, float *minorValue);
void DrawThermometer();
void DrawFlickThermometer();
void findMaxMin(float *fftImg_buffer,
		float *maxF, float *minF, float *offset,
		int logSwitch, int fftWidth, int fftHeight,
		int bkgVal);


void ctf_all_activate();

void phys_map_rotate(float *gold_buffer,
		     float *temp_buffer,
		     float theta, float phi, float omega);
void new_phys_map_rotate(float *gold_buffer,
			 float *temp_buffer,
			 float theta, float phi, float omega);

void calcCTF();
void calcMeanDefocus(float *meanValue);
int calcShortBackground();
void calcSizeFactor(float thisZoomFactor);
void evalSwitches(int argc, char **argv);
void readResourceFile();
void init_globals(int argc, char **argv);
void groundZero();
void XlibInitialize();
void s_error(char *problem_string);
int checkStatus(char *outStr);
int calcHistLength(int histVal);
void clipFileName(char *fName);
void stuffIt(unsigned char *buff, int len, unsigned char *intShort);
void stuffImage(unsigned short *thisBuffer, int i, int j, 
		int scaleIt, int fullScreenFirst);
void stuffMapImage(float *map_img_buf, 
		   float *thisBuffer, 
		   int ncols, 
		   int nrows, 
		   int nsecs,
		   int startImg,
		   int passedOffset);
void stuffPackedImage(int width,int height,
		      float **pkImg, int nimg,
		      int startImg);
void drawContour(float contourLvl, 
		 int destructive, 
		 int thisCurBox,
		 Widget thisWidget,
		 float *thisFloatBuf,
		 int deltaX, int deltaY,
		 BYTE *this_display_buffer);
void stuffContourBuffer();
int convertDisp2Image(int *inVal, int inOffset, int virOffset);
int convertImage2Disp(int *inVal, int inOffset, int virOffset);
void pushBox(int x, int y, int findThisBox, int *thisBox);
void popBox();
void removeBox(int x, int y, int diskCoords);
void pushRectBox(int button, int x, int y);
void pushPolyBox(int x, int y);
void popPolyBox();
void removePolyBox(int x, int y);
void filterFiles(char *filesStr, int tag);
void writeColormap(char *fName);
void readColormap(char *fName);
void pushColormap();
void popColormap();
void contrast_enhance(int min, int max);
void pointerLabelDraw(int x, int y);
void stuffDisplayBoxBuffer(int thisBox, char *filename, 
			   int thisZoomFactor);
void stuffRectangleBuffer(float *outBuf, char *filename);
void stuffDisplayMapBuffer(int thisSec, int thisZoomFactor, int boxOffset);
void stuffDisplayPackBoxBuffer(int thisSec, int thisZoomFactor,
			       int computeAppodize);
void handle_cr(Widget widget);
void writeBoxCoords(char *fName, int append);
void readBoxCoords(char *fName);
void readState(char *fName);
void writeState(char *fName);
void virtual_read_activate(int newX, int newY);
int askThem(char *infoStr, int systemModal, int dialogType);
int askThemAnswer(char *infoStr, int maxAnswer);
void read_batchjob(char *fName);
void read_param(char *fName, int readPackedFile);
void read_param_map(char *fName);
void read_param_scan(char *fName);
void write_param(char *fName);
void Single_Line(int whichParam,GC color);
void map2sfs(float resHiP);



void infoOK(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void infoCancel(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void write_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void read_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void exit_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void formResize(Widget  widget,
	    int     *tag,
	    XmDrawingAreaCallbackStruct *callback_data);
void toolMode_change(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void zoomText_change();
void main_zoom_activate();
void button_zoom_activate();
void plotWindow_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void raiseIt(Widget w);
void DrawWindow();
void DrawWindowExpose(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void drawingWindow_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void plotClick(Widget  widget,
	    int     *tag,
	    XmDrawingAreaCallbackStruct *callback_data);
void plotMotion(Widget	widget,
	caddr_t unused,   /* Client specified data */
	register XMotionEvent *pEvent);
void fftMotion(Widget	widget,
	caddr_t unused,   /* Client specified data */
	register XMotionEvent *pEvent);
void rotfltMotion(Widget	widget,
	caddr_t unused,   /* Client specified data */
	register XMotionEvent *pEvent);
void plotVertChange(Widget  widget,
		int     *tag,
		XmScrollBarCallbackStruct *callback_data);
void plotHorizChange(Widget  widget,
		int     *tag,
		XmScrollBarCallbackStruct *callback_data);
void plotScroll_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void cancel_fileSel(Widget	widget,
		int    *tag,
		XmFileSelectionBoxCallbackStruct *callback_data);
void gotFilename (Widget   w,
		  char *client_data,
		  XmFileSelectionBoxCallbackStruct *callback_data);
void imgNumText_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void imgNumText_changed(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void imgNumArrowActivate(Widget  widget,
	    int     *tag,
	    XmPushButtonCallbackStruct *callback_data);
void cmScrollArrowActivateButton();
void paintSizeArrowActivateButton();
void icosaArrowActivateButton();
void boxNumArrowActivateButton();
void boxInitArrowActivateButton();
void imgNumArrowActivateButton();
void fftImgNumArrowButton();
void radiusArrowActivateButton();
void taArrowActivateButton();
void ta2ArrowActivateButton();
void gbsArrowActivateButton();
void ppinArrowButton();
void ppintext_changed();
void binPPArrowButton();
void binPPtext_changed();
void stepPPArrowButton();
void stepPPtext_changed();
void radiusText_create();
void radiusText_changed();
void radiusArrowActivate();
void fft_step_changed();
void diffSec_really_doit();
void mapShade_activate_work();
void boxInitActivate(Widget widget, 
		     int *tag, 
		     XmAnyCallbackStruct *callback_data);
void imgNumActivate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void colormap_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void colormap_ramp_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void colormap_ramp_change(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void DrawColormapWindow();
void DrawTestColormapWindow(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void colorScrollChange(Widget  widget,
		int     *tag,
		XmScrollBarCallbackStruct *callback_data);
void colorClick(Widget	widget,
	int    *tag,
	XmDrawingAreaCallbackStruct *draw_data);
void colormapValue_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void colormapMotion(Widget	widget,
		caddr_t unused,   /* Client specified data */
		register XMotionEvent *pEvent);
void circAvgMotion(Widget	widget,
		caddr_t unused,   /* Client specified data */
		register XMotionEvent *pEvent);
void tog_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void lock_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void toggle_change(Widget  widget,
		int     *tag,
		XmToggleButtonCallbackStruct *callback_data);
void options_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void squareText_changed();
void boxMode_change(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void featherMode_change(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void backgroundMode_change(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void dispMode_change(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void histMode_change(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void histogram_activate();
void histArrowActivate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void mapMovie_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void mapShade_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void mapMovie_show_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void editBox_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void editBoxForm_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void editBox_undo_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void editBox_save_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void DrawEditBoxWindow(int mapMovie);
void editBoxClick(Widget	widget,
		int    *tag,
		XmDrawingAreaCallbackStruct *draw_data);
void editBoxMotion(Widget	widget,
		caddr_t unused,   /* Client specified data */
		register XMotionEvent *pEvent);
void editBoxText_changed(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data); 
void editBoxZoomText_changed(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void editBoxPaintArrowActivate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void editBoxZoomArrowActivate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void editBoxArrowActivate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void editBoxPaintText_changed(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void editBoxText_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void rulerText_changed(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void rulerText_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void help_text_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void help_save_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void help_screen_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void context_help_screen_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void setFont_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void tryThis_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void angleText_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void surfDensValueChange(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void surfDensScrollChange(Widget  widget,
	    int     *tag,
	    XmScrollBarCallbackStruct *callback_data);
void surfDensValue_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void surfDensScroll_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void subMapShadeScreen_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void mapShadeScreen_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void stats_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void stat_text_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void ref_select_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void ref_text_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void label_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void stepMode_change(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void stepMode_button_change(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void stepMode_create(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void movieMinMax_change(Widget  widget,
	    int     *tag,
	    XmScrollBarCallbackStruct *callback_data);
void correlate_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data); 
void generic_deactivate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);

void ref_correlate_activate();
void autoBox_activate();
void DrawCircAvgWindow();
void circAvg_print_activate();
void circAvg_pifOut_activate();
void circAvg_change();
void circAvg_screen_activate();
static void circAvg_list_proc();
void circAvgText_create();
void circAvgText_changed();
void circAvgArrowActivate();
void flickArrow_create();
void flickArrowActivate();
void movieArrow_create();
void mapMovieArrowActivate();
void autoDR_update_activate();
void autoDR_activate();
void auto2DDR_curves_activate();
void auto2DDR_update_activate();
void auto2DDR_ctflines_activate();
void auto2DDR_scurve_activate();
void auto2DDR_drawmodified_activate();
void auto2DDR_activate();
void model_clear_activate();
void model_undo_activate();
void model_save_activate();
void modelObject_change();
void adr_toggle_change();
void fix2D_flicker_activate();
void fft_flicker_activate();
void fft_activate();
void read_fftbuffer_activate();
void fft_writeFB_activate();
void fft_1DCA_activate();
void fft_change();
void fft_change_toggle();
void fft_create();
void DrawFFTWindow();
void latText_create();
void latText_changed();
void calcLattice();
void fftClick();
void ctfText_changed();
void contourWindow_create();
void fftArrowActivate();
void icosaArrowActivate();
void samRenArrowActivate();
void adrab_reset_activate();
void latReset_activate();
void latScrollChanged();
void latTextChanged();
void ctfTextChanged();
void lohiRadiusChanged();
void lopTextChanged();
void hipTextChanged();
void impose_changed();
void refine_changed();
void movePhaseOrigin_activate();
void backSize_changed();
void paramFileRead_work();
void fourAver_work();
void fourAver_write_sf();
void fft_average_activate();
void fftImgNum_cr_activate();
void rotflt_arrow_activate();
void rotflt_arrow23_activate();
void rotflt_create();        	
void rotflt_write_activate();
void rotflt_change();        	
void rotflt_screen_activate();
void DrawROTFLTWindow();
void rotfltClick();
void scaleText_changed();
void rotflt_tryit_activate();
void rotflt_forceTryit_activate();
void coravg_scroll_change();
void emcoravg_text_changed();
void emcoravg_arrow_activate();
void stuffEmcoravgImage();
void stuffImgconvImage();
void read_imgconv(char *fName);
void read_emcoravg(char *fName);
void DrawEmcoravgWindow();
void emcoravg_create();
void emcoravg_file_activate();
void emcoravg_tryit_activate();
void emcoravg_save_activate();
void emcoravg_screen_activate();
void crysDisp_create();   
void crysDisp_screen_activate();
void crysDisp_tryit_activate();
void crystalDisp_changed();
void crysDisp_scroll_change();
void foldAxis_activate();
void crysDisp_cr_activate();
void crysDisp_reset_activate();
void cDispSymm_changed();
void cDisp_zoom_activate();
void cDisp_newAdimBdim_changed();
void mainCdisp_zoom_activate();
void addAverage_activate();        
void binning_activate();        
void binning_screen_activate();
void interpolate_activate();        
void interp_screen_activate();
void model_activate();        
void model_screen_activate();
void mb_scroll_change();
void fourierFilt_activate();        
void fourierFilt_work();        
void checkBkg_activate();        
void normalize_activate();        
void normalize_activate_2();        
void linGrad_activate();
void unblob_create();		
void unblob_screen_activate();
void unblob_scroll_change();
void unblob_tryit_activate();
void unblob_reset_activate();
void unblob_save_activate();
void unblob_create_label();
void unblob_text_change();
void reverse_sections();
void destructive_reverse_sections();
void fixit_create();             
void fixit_changed();
void fixit_activate();
void fft_mode_activate();
void fftForm_create();
void ctfArrowActivate();
void ctfCurve_cr_activate();
void ctf_ioparms_activate();
void surfDens_CR();
void crysDisp_contour_activate();
void movie_contour_activate();
void movie_contour_state();
void histEqualize();
void save_title_activate();
void calcSF_work(int thisCalcSFs);
void calcSF_activate();
void smMinMax_reset_activate();
void flicker_change();
void coravgArrowActivate();
void coravgArrowButton();
void phaseArrowActivate();
void phaseArrowButton();
void smMMaxArrowActivate();
void smMMaxArrowButton();
void subtractMap_doit();
void subtractMap_2d3d_activate();
void subtractMap_activate();
void subtractMap_work();
void subtractMap_cc();
void subtractMap_findRad();
void subtractMap_computeScaling();
void subtractMap_findRad_disp();
void subtractMap_findMag_disp();
void subtractMap_read();
void subtractMap_browse_activate();
void subtractMap_flicker_arm();
void subtractMap_flicker_disarm();
void subtractMap_diffSection();
void drawMinMaxRadius();
void diffSec_scroll_change();
void diffSec_do();
void diffSecSave_activate();
void radTog_change();
void rescale_activate();
void rescale_densities();
void invert_activate();
void taArrowActivate();
void ta2ArrowActivate();
void gbsArrowActivate();
void truncadd_half_activate();
void truncadd_changed();                 
void truncadd_activate();                 
void truncadd_do_activate();
void rotate_activate();                 
void rotate_do_activate();
void rotate_create();
void removeBlemish_activate();                 
void fix_activate();                 
void fix_change();
void fix_create();
void fix_do_activate();
void fix2D_do_activate();
void fix2D_create();
void fftMeasWrite_activate();
void point_create();
void point_color_activate();
void drawPointScreen();
float findRealValue();
void stat_calc_activate();
void pifhead_activate();
void appod_create();
void appodize_activate();
void boxgrid_activate();
void boxgrid2_activate();
void boxcolor_change();
void box_change();
void autobox_scroll_change();
void colormap_scroll_activate();
void drawLiteActivate();
void drawFFTLiteActivate();
void version_scroll_activate();
void version_activate();
void versionArrowActivate();
void ppMinMax_scroll_change();
void ppinArrowActivate();
void binPPArrowActivate();
void stepPPArrowActivate();
void projASU_activate();
void partproc_activate();              
void partproc_sort_activate();
void partproc_proj_activate();
void cavgPP_change();
void cavgPP_do_activate();
void cavgPP_activate();
void binning_change();
void manipPP_change();
void manipPP_activate();
void partproc_manip_work();
void partproc_delta_work();
void partproc_delta_activate();
void partproc_rebox_activate();
void partproc_deltaomega_activate();
void partproc_deltaomega_work();
void partproc_deltaoldNew_work();
void batch_focus();
void batchjob_allasu_activate();
void batchjob_submit_activate();
void batchjob_activate();
void batchparam_list_proc();
void partproc_write_activate();
void text_create();
void paramFile_browse_activate();
void paramFile_read_activate();
void paramScanFile_read_activate();
void projContour_cr_activate();
void projasu_cr_activate();
void project_one();
void paramMapFile_read_activate();
void icosWindow_create();
void unitCell_activate();
void xyPlot_activate();
void plotCavgPP_activate();
void projASUunitCell_work();
void ico_hist_activate();
void DrawICOSWindow();
void DrawIcosahedron(float xrot,float yrot,float zrot,
		     int xoffset, int yoffset, 
		     float scaleFactor, Window thisWin);
void findNearestParticle(int *x, int *y,
			 P_INFO **foundPart,
			 int toggleFound,
			 int makeThisParticle,
			 int showPackedBox,
			 int buttonNum);
void convertThetaPhi(int x, int y, 
		     float *thisTheta, float *thisPhi);
void convertXY(int *convX, int *convY, float fX, float fY);
void icosaMotion(Widget	widget,
	caddr_t unused,   /* Client specified data */
	register XMotionEvent *pEvent);
void icosaClick();
void markXYplotOutliers();
void ppMinMax_change();
void ctfValue_change();
void paramMode_change();
void symmetry_activate();
void movement_activate();
void views_activate();
void diamText_change();
void find_nearest_neighbors(INFO *head);
void info_list_create();
void info_list_proc();
void resolution_tryit_activate();
void fourAver_manage();
void fourAver_resetSfs();
void ctf_reset_activate();
void ctf_focus();
void ctf_appodize_activate();
void rotate_xyz_activate();
void autoDwritePIFhead_activate();
void autoDefocusRefine_activate();
void auto2DDefocusRefine_activate();
void paramFileGenerate_activate();
void paramFileRead_activate();


/* extern void sleep(); */
/* extern pid_t fork(); */
extern void vt_cls();  

void mainRobem_sub_UNIX(unsigned short **int_buffer, 
			fString_t *fName,
			int *i, int *j); 

extern int checkMemory(void *status, int line, char *file);

extern void rad_dens_fast_rob_C (float *coremap, 
			  int ncol, int nrow, int ndepth, 
			  float *buf,
			  float x_org, float y_org, float z_org, 
			  float radius, float smallestDens,
			  float theta, float phi,float omega,
			  int colOffset, int rowOffset,int largestDim,
			  int thisIrev);

extern void icosRadialProject (float *coremap, 
			       int ncol, int nrow, int ndepth, 
			       float *buf,
			       float x_org, float y_org, float z_org, 
			       float radius, float smallestDens,
			       float theta, float phi,float omega,
			       int colOffset, int rowOffset,int largestDim,
			       int thisIrev, int tag,
			       float thresh, float slid, int destroy,
			       float minRadius);

extern int callXdy (double x, double y, double z, 
		     double minRad, double maxRad);

extern void thetaPhi_dphshd_C (float *coremap,
			       float *origMap,
			       int ncol, int nrow, 
			       float *dphshd_map, 
			       BYTE shift, BYTE surf, 
			       int sec1, int sec2, 
			       int col1, int col2,
			       int row1, int row2,
			       int num_views,
			       int thisIrev, float surf_dens,
			       int useForMinMax,
			       float *shad_map,
			       double phi, double theta, double omega,
			       double minRadius, double maxRadius,
			       double xCen, double yCen, double zCen,
			       float radialXcen, float radialYcen, 
			       float radialZcen,
			       int lowRes, int topRadialSec, int step);

extern double getVoxel(float *coremap,
		       double thisCol, double thisRow, double thisSecn,
		       double theta, double phi, double omega,
		       int ncol, int nrow);

extern void newTranslateThetaPhiOmega(double *thisCol,
				      double *thisRow,
				      double *thisSecn,
				      double xCen,double yCen,double zCen,
				      int col,int row,int secn,
				      double t1,
				      double t2,
				      double t3,
				      double t4,
				      double t5,
				      double t6,
				      double t7,
				      double t8,
				      double t9
				      );

extern void ricardo_routine();

extern void emmap_rob_dphshd_C (float *map,
				float *final_map,
				float *radial_map,
				float *shad_map,
				float *origMap,
				int   numFrames,
				float surfaceDensity,
				int ncol, int nrow, 
				int firstStep, int nsec,
				int col1, int col2,
				int row1, int row2,
				char ikey,
				double phi, double thetaY,
				double minRadius, double maxRadius,
				double xCen, double yCen, double zCen,
				float radialXcen, float radialYcen, 
				float radialZcen,
				int lowRes, int topRadialSec);
extern void img_write_recl_open(fString_t *fileName, int *lun, 
				short *int_image_buffer1,
				int *image_width, int *image_height);
#ifdef sgi
/* DOn't do it */
#else
#endif

extern void   yji_ctf1dfit_( int *ncol, int *nrow, int *nimg, 
			     int *fftSize, float *pixelSize,
			     float *volts, float *res_hi, float *defMajor, 
			     float *ampContrast, float *Cs, 
			     float *allPackImg, float *chisq, 
			     float *a,
			     float *adr_s,
			     float *adr_sminusb,
			     float *adr_ctfsq,
			     float *adr_d,
			     float *adr_b,
			     float *ca1d,
			     int   *calcIt,
			     float *saveFftAvg
			     );
extern char *img_filename;
extern void img_open();
extern void img_pack_find_prompt();
extern void fft_latref2_index_ ();
extern void fft_latref2_spot_ ();
extern void yongning_routine_();
extern void emsf_backt_rob_(int *sf_type,
			    int *em_ndim,
			    int *nsf,
			    strucFact_t *sFacs,
			    float *map_min, float *map_max,
			    float *aoverb, 
			    float *alp_star, float *bet_star, 
			    float *gam_star,
			    float *a_star, float *b_star, 
			    float *c_star,
			    float *vol, float *vol_star, 
			    float *a_cell, float *b_cell, 
			    float *c_cell, 
			    float *abang,
			    float *alp,float *bet, float *gam,
			    float *float_image_buffer1,
			    int *backSizeA,int *backSizeB);
extern void emsf_origins_rob_(int *em_ndim,int *sf_type,int *nsf,
			      float *sf_sumfsq, 
			      strucFact_t *sFacs, int *ikey);
extern void emsf_sym_rob_(int *em_ndim, int *sf_type, int *nlim, int *nsf, 
			 strucFact_t *sFacs,
			 float *gamstr, 
			 int *ikey, float *thisAoverb);
extern void map_rot_();
extern void img_fft_fill_rob_();
extern void img_fft_fill_rob_char_();
extern void img_filt_();
extern void fft_2d_();
extern void fft_dsp_rob_();
extern void fft_fltmsk_rob_();
extern void fft_2d_rob_();
extern void fft_setdim_local_();
extern void fft_copy1_();
extern void latcen_();
extern void latcen2_();
extern void emrotflt_main_(float *powSpec,
			   float *elipX,float *elipY,
			   int *irad,
			   int *idim1Rotflt,
			   float *phi0,float *ascal,float *bscal,
			   float *twodd,int *jzero,
			   int *radiusM1,int *max_bo,
			   float *fft_step,int *nannuli,
			   float *del_br,
			   float *rho,
			   float *del_lr,
			   int *iopt,
			   int *lg_start,int *lg_end,
			   float *xorig,float *yorig,
			   float *pow_scale, int *nrot,
			   int *numCols, int *numRows,
			   float *sump1, float *sump2, float *sump3,
			   float *fft_origx, float *fft_origy, 
			   float *winScale, float *sigma,
			   float *tempRotflt_buffer,
			   int *newStart, float *sigMax, float *sigMaxTot
			   );
extern void mapdsp_gplot_rob_(float *crys_float_image_buffer, 
			      unsigned short *disp_buffer, 
			      float *aDscale, 
			      int *cdNumCols, int *cdNumRows, int *cdNumSecs, 
			      int *npixx, int *npixy, 
			      float *xstrt, float *ystrt, 
			      int *ny, 
			      int *ncellx, int *ncelly, 
			      int *section,
			      float *new_min,
			      float *angle,
			      float *aoverbPass,
			      int *plotX,int *plotY,
			      int *adim, int *bdim, float *passGamma);
extern void img_lingrad_rob_(float *img, int *ncol, int *nrow,
			     int *irad, float *imgBkg, float *expFac);
extern void count_kill_blobs_rob_(int *bCnt, int *uniqB,
				  float *contMin, float *contMax,
				  int *ublobSiz,
				  int *killFlag, 
				  float *killValue, 
				  float *bgMin, float *bgMax,
				  int *status,
				  float *unblobBuf,
				  int *nc, int *nr, int *ns,
				  blobStructure_t *bL, int *radUB, int *invU
				  );
extern void GetBlobSizeLimit_rob_();
extern void getblobminmaxint_rob_();
extern void contur_a_rob_();
extern void contur_b_rob_();
extern void contur_c_rob_();
extern void contur_d_rob_();
extern void rad_dens_fast_rob_();
extern void ccf_refine_();
extern void ref_ccf_refine_();
extern void fft_avg_ref_();
extern void map_prj_();
extern void map_scale_();
extern void main_map3dt_();
extern void map3dt_info_rwa_();
extern void main_emsf_();
extern void main_emsf2_();
extern float wavel_();
extern void ccomap3ds_();
extern void cc_3d_(float *float_image_buffer1,
		   float *subtract_image_buffer1,
		   int *numCols, int *numRows, int *numSecs,
		   int *sub2binCol, int *sub2binRow, int *sub2binSec,
		   int *ichc,float *std,
		   float *xc, float *yc, float *zc,
		   float *magini,float *magstep,int *magchk,
		   float *radlo,float *radhi,float *radstep,float *bmag,
		   int *iix,int *iiy,int *iiz,
		   float *mags, float *coefs,
		   int *subMap2d3d, int *qtr_switch
		   );
extern void ccomap3cs_(float *thisBuf1,
		       float *thisBuf2,
		       int *numCols,int *numRows,int *sec1,
		       float *tbccoef,
		       int *sub2binCol,int *sub2binRow,int *sec2,
		       int *nfit,
		       float *cntr1,float *cntr2,
		       int *lpol,float *map_std,
		       float *xc,float *yc,float *zc,
		       float *radlo,float *radhi,float *radstep,
		       float *bmag,
		       float *rdens,float *wrdens,float *ccoef,
		       float *wccoef,float *s_wccoef,int *nbins,
		       float *diffSecScrollValue,
		       float *diffSec2ScrollValue,
		       int *slidingWindow,
		       int *qtr_switch, int *subMap2d3d,
		       float *threshScrollValue,
		       int *threshAbs
		       );
extern void compute_scaling_();
extern void mag_write_();
extern void fft_lop_();
extern void fft_hip_();

extern void img_fill();
extern void copy_r4();
extern void fft_pshift();
extern void emfft_spt_rob1();
extern void emfft_spt_rob2();
extern void rob_sf_pshift();
extern void rec_to_real();
extern void fft_fltllmsk_rob();
extern void img_stats_rob();
extern void apply_ctf_rtn();
void fft_dsp_c(float *fft, BYTE *img, 
	       int idim1, int idim2,
	       int ncol, int nrow, 
	       float thresh_hi, float thresh_lo,
	       int hiPixel, int loPixel, 
	       int linear, int amps);
void fft_feedback_activate();
void dio_change();
void apNewOrig_do_activate();
void doTheCdispZoom(Widget widget, float speedFactor);
extern void convert_abpart(float *apart, float *bpart,
			   float *amp, float *phase);
void imgconv_create();
void imgconv_screen_activate();
void imgconv_tryit_activate();
void imgconv_file_activate();


/* The names and addresses of things that Mrm.has to bind.  The names do
 * not have to be in alphabetical order.  */

static MrmRegisterArg reglist[] = {
    {"imgconv_create", (caddr_t) imgconv_create},
    {"imgconv_screen_activate", (caddr_t) imgconv_screen_activate},
    {"imgconv_tryit_activate", (caddr_t) imgconv_tryit_activate},
    {"imgconv_file_activate", (caddr_t) imgconv_file_activate},
    {"apNewOrig_do_activate", (caddr_t) apNewOrig_do_activate},
    {"fft_feedback_activate", (caddr_t) fft_feedback_activate},
    {"paramFileGenerate_activate", (caddr_t) paramFileGenerate_activate},
    {"paramFileRead_activate", (caddr_t) paramFileRead_activate},
    {"autoDwritePIFhead_activate", (caddr_t) autoDwritePIFhead_activate},
    {"autoDefocusRefine_activate", (caddr_t) autoDefocusRefine_activate},
    {"auto2DDefocusRefine_activate", (caddr_t) auto2DDefocusRefine_activate},
    {"rotate_xyz_activate", (caddr_t) rotate_xyz_activate},
    {"impose_changed", (caddr_t) impose_changed},
    {"refine_changed", (caddr_t) refine_changed},
    {"movePhaseOrigin_activate", (caddr_t) movePhaseOrigin_activate},
    {"backSize_changed", (caddr_t) backSize_changed},
    {"fourAver_write_sf", (caddr_t) fourAver_write_sf},
    {"fourAver_resetSfs", (caddr_t) fourAver_resetSfs},
    {"fourAver_manage", (caddr_t) fourAver_manage},
    {"info_list_create", (caddr_t) info_list_create},
    {"info_list_proc", (caddr_t) info_list_proc},
    {"diamText_change", (caddr_t) diamText_change},
    {"views_activate", (caddr_t) views_activate},
    {"movement_activate", (caddr_t) movement_activate},
    {"symmetry_activate", (caddr_t) symmetry_activate},
    {"paramMode_change", (caddr_t) paramMode_change},
    {"ppMinMax_change", (caddr_t) ppMinMax_change},
    {"ctfValue_change", (caddr_t) ctfValue_change},
    {"icosaClick", (caddr_t) icosaClick},
    {"DrawICOSWindow", (caddr_t) DrawICOSWindow},
    {"unitCell_activate", (caddr_t) unitCell_activate},
    {"plotCavgPP_activate", (caddr_t) plotCavgPP_activate},
    {"xyPlot_activate", (caddr_t) xyPlot_activate},
    {"ico_hist_activate", (caddr_t) ico_hist_activate},
    {"icosWindow_create", (caddr_t) icosWindow_create},
    {"projContour_cr_activate", (caddr_t) projContour_cr_activate},
    {"project_one", (caddr_t) project_one},
    {"projasu_cr_activate", (caddr_t) projasu_cr_activate},
    {"paramScanFile_read_activate", (caddr_t) paramScanFile_read_activate},
    {"paramMapFile_read_activate", (caddr_t) paramMapFile_read_activate},
    {"paramFile_read_activate", (caddr_t) paramFile_read_activate},
    {"paramFile_browse_activate", (caddr_t) paramFile_browse_activate},
    {"text_create", (caddr_t) text_create},
    {"batchparam_list_proc", (caddr_t) batchparam_list_proc},
    {"batchjob_allasu_activate", (caddr_t) batchjob_allasu_activate},
    {"batchjob_submit_activate", (caddr_t) batchjob_submit_activate},
    {"batchjob_activate", (caddr_t) batchjob_activate},
    {"batch_focus", (caddr_t) batch_focus},
    {"projASU_activate", (caddr_t) projASU_activate},
    {"cavgPP_change", (caddr_t) cavgPP_change},
    {"cavgPP_do_activate", (caddr_t) cavgPP_do_activate},
    {"cavgPP_activate", (caddr_t) cavgPP_activate},
    {"binning_change", (caddr_t) binning_change},
    {"manipPP_change", (caddr_t) manipPP_change},
    {"manipPP_activate", (caddr_t) manipPP_activate},
    {"partproc_manip_work", (caddr_t) partproc_manip_work},
    {"partproc_delta_work", (caddr_t) partproc_delta_work},
    {"partproc_delta_activate", (caddr_t) partproc_delta_activate},
    {"partproc_deltaomega_activate", (caddr_t) partproc_deltaomega_activate},
    {"partproc_deltaomega_work", (caddr_t) partproc_deltaomega_work},
    {"partproc_deltaoldNew_work", (caddr_t) partproc_deltaoldNew_work},
    {"partproc_rebox_activate", (caddr_t) partproc_rebox_activate},
    {"partproc_proj_activate", (caddr_t) partproc_proj_activate},
    {"partproc_sort_activate", (caddr_t) partproc_sort_activate},
    {"partproc_write_activate", (caddr_t) partproc_write_activate},
    {"partproc_activate", (caddr_t) partproc_activate},
    {"ppMinMax_scroll_change", (caddr_t) ppMinMax_scroll_change},
    {"ppinArrowActivate", (caddr_t) ppinArrowActivate},
    {"binPPArrowActivate", (caddr_t) binPPArrowActivate},
    {"stepPPArrowActivate", (caddr_t) stepPPArrowActivate},
    {"versionArrowActivate", (caddr_t) versionArrowActivate},
    {"version_scroll_activate", (caddr_t) version_scroll_activate},
    {"version_activate", (caddr_t) version_activate},
    {"drawLiteActivate", (caddr_t) drawLiteActivate},
    {"drawFFTLiteActivate", (caddr_t) drawFFTLiteActivate},
    {"colormap_scroll_activate", (caddr_t) colormap_scroll_activate},
    {"autobox_scroll_change", (caddr_t) autobox_scroll_change},
    {"main_zoom_activate", (caddr_t) main_zoom_activate},
    {"point_create", (caddr_t) point_create},
    {"point_color_activate", (caddr_t) point_color_activate},
    {"appod_create", (caddr_t) appod_create},
    {"appodize_activate", (caddr_t) appodize_activate},
    {"boxgrid_activate", (caddr_t) boxgrid_activate},
    {"boxgrid2_activate", (caddr_t) boxgrid2_activate},
    {"boxcolor_change", (caddr_t) boxcolor_change},
    {"box_change", (caddr_t) box_change},
    {"fftMeasWrite_activate", (caddr_t) fftMeasWrite_activate},
    {"save_title_activate", (caddr_t) save_title_activate},
    {"fix_change", (caddr_t) fix_change},
    {"fix_create", (caddr_t) fix_create},
    {"fix_do_activate", (caddr_t) fix_do_activate},
    {"fix2D_do_activate", (caddr_t) fix2D_do_activate},
    {"fix2D_create", (caddr_t) fix2D_create},
    {"removeBlemish_activate", (caddr_t) removeBlemish_activate},
    {"fix_activate", (caddr_t) fix_activate},
    {"radTog_change", (caddr_t) radTog_change},
    {"diffSecSave_activate", (caddr_t) diffSecSave_activate},
    {"diffSec_scroll_change", (caddr_t) diffSec_scroll_change},
    {"adrab_reset_activate", (caddr_t) adrab_reset_activate},
    {"smMinMax_reset_activate", (caddr_t) smMinMax_reset_activate},
    {"flicker_change", (caddr_t) flicker_change},
    {"coravgArrowActivate", (caddr_t) coravgArrowActivate},
    {"phaseArrowActivate", (caddr_t) phaseArrowActivate},
    {"smMMaxArrowActivate", (caddr_t) smMMaxArrowActivate},
    {"smMMaxArrowActivate", (caddr_t) smMMaxArrowActivate},
    {"subtractMap_2d3d_activate", (caddr_t) subtractMap_2d3d_activate},
    {"subtractMap_diffSection", (caddr_t) subtractMap_diffSection},
    {"subtractMap_flicker_disarm", (caddr_t) subtractMap_flicker_disarm},
    {"subtractMap_flicker_arm", (caddr_t) subtractMap_flicker_arm},
    {"subtractMap_activate", (caddr_t) subtractMap_activate},
    {"subtractMap_cc", (caddr_t) subtractMap_cc},
    {"subtractMap_findRad", (caddr_t) subtractMap_findRad},
    {"subtractMap_findRad_disp", (caddr_t) subtractMap_findRad_disp},
    {"subtractMap_findMag_disp", (caddr_t) subtractMap_findMag_disp},
    {"subtractMap_computeScaling", (caddr_t) subtractMap_computeScaling},
    {"subtractMap_work", (caddr_t) subtractMap_work},
    {"subtractMap_browse_activate", (caddr_t) subtractMap_browse_activate},
    {"calcSF_activate", (caddr_t) calcSF_activate},
    {"rescale_activate", (caddr_t) rescale_activate},
    {"rescale_densities", (caddr_t) rescale_densities},
    {"invert_activate", (caddr_t) invert_activate},
    {"taArrowActivate", (caddr_t) taArrowActivate},
    {"ta2ArrowActivate", (caddr_t) ta2ArrowActivate},
    {"gbsArrowActivate", (caddr_t) gbsArrowActivate},
    {"truncadd_half_activate", (caddr_t) truncadd_half_activate},
    {"truncadd_activate", (caddr_t) truncadd_activate},
    {"truncadd_do_activate", (caddr_t) truncadd_do_activate},
    {"truncadd_changed", (caddr_t) truncadd_changed},
    {"rotate_activate", (caddr_t) rotate_activate},
    {"rotate_do_activate", (caddr_t) rotate_do_activate},
    {"rotate_create", (caddr_t) rotate_create},
    {"ref_select_activate", (caddr_t) ref_select_activate},
    {"ref_text_create", (caddr_t) ref_text_create},
    {"crysDisp_contour_activate", (caddr_t) crysDisp_contour_activate},
    {"movie_contour_state", (caddr_t) movie_contour_state},
    {"movie_contour_activate", (caddr_t) movie_contour_activate},
    {"ctf_appodize_activate", (caddr_t) ctf_appodize_activate},
    {"ctf_reset_activate", (caddr_t) ctf_reset_activate},
    {"ctf_focus", (caddr_t) ctf_focus},
    {"ctf_ioparms_activate", (caddr_t) ctf_ioparms_activate},
    {"ctfCurve_cr_activate", (caddr_t) ctfCurve_cr_activate},
    {"ctfArrowActivate", (caddr_t) ctfArrowActivate},
    {"histMode_change", (caddr_t) histMode_change},
    {"fftForm_create", (caddr_t) fftForm_create},
    {"adr_toggle_change", (caddr_t) adr_toggle_change},
    {"autoDR_update_activate", (caddr_t) autoDR_update_activate},
    {"autoDR_activate", (caddr_t) autoDR_activate},
    {"auto2DDR_curves_activate",(caddr_t)auto2DDR_curves_activate},
    {"auto2DDR_ctflines_activate",(caddr_t)auto2DDR_ctflines_activate},
    {"auto2DDR_scurve_activate",(caddr_t)auto2DDR_scurve_activate},
    {"auto2DDR_drawmodified_activate",(caddr_t)auto2DDR_drawmodified_activate},
    {"auto2DDR_update_activate", (caddr_t) auto2DDR_update_activate},
    {"auto2DDR_activate", (caddr_t) auto2DDR_activate},
    {"mb_scroll_change", (caddr_t) mb_scroll_change},
    {"model_clear_activate", (caddr_t) model_clear_activate},
    {"model_undo_activate", (caddr_t) model_undo_activate},
    {"model_save_activate", (caddr_t) model_save_activate},
    {"modelObject_change", (caddr_t) modelObject_change},
    {"fix2D_flicker_activate", (caddr_t) fix2D_flicker_activate},
    {"fft_flicker_activate", (caddr_t) fft_flicker_activate},
    {"fft_mode_activate", (caddr_t) fft_mode_activate},
    {"fixit_activate", (caddr_t) fixit_activate},
    {"fixit_create", (caddr_t) fixit_create},
    {"fixit_changed", (caddr_t) fixit_changed},
    {"reverse_sections", (caddr_t) reverse_sections},
    {"destructive_reverse_sections", (caddr_t) destructive_reverse_sections},
    {"unblob_save_activate", (caddr_t) unblob_save_activate},
    {"unblob_reset_activate", (caddr_t) unblob_reset_activate},
    {"unblob_tryit_activate", (caddr_t) unblob_tryit_activate},
    {"unblob_scroll_change", (caddr_t) unblob_scroll_change},
    {"unblob_screen_activate", (caddr_t) unblob_screen_activate},
    {"unblob_create_label", (caddr_t) unblob_create_label},
    {"unblob_create", (caddr_t) unblob_create},
    {"unblob_text_change", (caddr_t) unblob_text_change},
    {"linGrad_activate", (caddr_t) linGrad_activate},
    {"binning_screen_activate", (caddr_t) binning_screen_activate},
    {"binning_activate", (caddr_t) binning_activate},
    {"interp_screen_activate", (caddr_t) interp_screen_activate},
    {"interpolate_activate", (caddr_t) interpolate_activate},
    {"model_screen_activate", (caddr_t) model_screen_activate},
    {"model_activate", (caddr_t) model_activate},
    {"addAverage_activate", (caddr_t) addAverage_activate},
    {"fourierFilt_activate", (caddr_t) fourierFilt_activate},
    {"fourierFilt_work", (caddr_t) fourierFilt_work},
    {"checkBkg_activate", (caddr_t) checkBkg_activate},
    {"normalize_activate", (caddr_t) normalize_activate},
    {"normalize_activate_2", (caddr_t) normalize_activate_2},
    {"foldAxis_activate", (caddr_t) foldAxis_activate},
    {"coravg_scroll_change", (caddr_t) coravg_scroll_change},
    {"emcoravg_text_changed", (caddr_t) emcoravg_text_changed},
    {"coravgArrowActivate", (caddr_t) coravgArrowActivate},
    {"DrawEmcoravgWindow", (caddr_t) DrawEmcoravgWindow},
    {"emcoravg_create", (caddr_t) emcoravg_create},
    {"emcoravg_file_activate", (caddr_t) emcoravg_file_activate},
    {"emcoravg_save_activate", (caddr_t) emcoravg_save_activate},
    {"emcoravg_tryit_activate", (caddr_t) emcoravg_tryit_activate},
    {"emcoravg_screen_activate", (caddr_t) emcoravg_screen_activate},
    {"cDisp_newAdimBdim_changed", (caddr_t) cDisp_newAdimBdim_changed},
    {"mainCdisp_zoom_activate", (caddr_t) mainCdisp_zoom_activate},
    {"cDispSymm_changed", (caddr_t) cDispSymm_changed},
    {"crysDisp_reset_activate", (caddr_t) crysDisp_reset_activate},
    {"crysDisp_scroll_change", (caddr_t) crysDisp_scroll_change},
    {"crystalDisp_changed", (caddr_t) crystalDisp_changed},
    {"crysDisp_screen_activate", (caddr_t) crysDisp_screen_activate},
    {"crysDisp_tryit_activate", (caddr_t) crysDisp_tryit_activate},
    {"crysDisp_create", (caddr_t) crysDisp_create},
    {"rotflt_arrow_activate", (caddr_t) rotflt_arrow_activate},
    {"rotflt_change", (caddr_t) rotflt_change},
    {"rotflt_tryit_activate", (caddr_t) rotflt_tryit_activate},
    {"rotflt_forceTryit_activate", (caddr_t) rotflt_forceTryit_activate},
    {"scaleText_changed", (caddr_t) scaleText_changed},
    {"DrawROTFLTWindow", (caddr_t) DrawROTFLTWindow},
    {"rotfltClick", (caddr_t) rotfltClick},
    {"rotflt_write_activate", (caddr_t) rotflt_write_activate},
    {"rotflt_create", (caddr_t) rotflt_create},
    {"rotflt_screen_activate", (caddr_t) rotflt_screen_activate},
    {"fft_average_activate", (caddr_t) fft_average_activate},
    {"fourAver_work", (caddr_t) fourAver_work},
    {"latReset_activate", (caddr_t) latReset_activate},
    {"icosaArrowActivate", (caddr_t) icosaArrowActivate},
    {"samRenArrowActivate", (caddr_t) samRenArrowActivate},
    {"fftArrowActivate", (caddr_t) fftArrowActivate},
    {"contourWindow_create", (caddr_t) contourWindow_create},
    {"ctfText_changed", (caddr_t) ctfText_changed},
    {"latText_changed", (caddr_t) latText_changed},
    {"latText_create", (caddr_t) latText_create},
    {"fft_1DCA_activate", (caddr_t) fft_1DCA_activate},
    {"fft_create", (caddr_t) fft_create},
    {"fft_change", (caddr_t) fft_change},
    {"fft_change_toggle", (caddr_t) fft_change_toggle},
    {"fft_activate", (caddr_t) fft_activate},
    {"read_fftbuffer_activate", (caddr_t) read_fftbuffer_activate},
    {"fft_writeFB_activate", (caddr_t) fft_writeFB_activate},
    {"circAvg_print_activate", (caddr_t) circAvg_print_activate},
    {"circAvg_pifOut_activate", (caddr_t) circAvg_pifOut_activate},
    {"circAvg_change", (caddr_t) circAvg_change},
    {"circAvgText_create", (caddr_t) circAvgText_create},
    {"circAvgText_changed", (caddr_t) circAvgText_changed},
    {"circAvgArrowActivate", (caddr_t) circAvgArrowActivate},
    {"flickArrow_create", (caddr_t) flickArrow_create},
    {"flickArrowActivate", (caddr_t) flickArrowActivate},
    {"movieArrow_create", (caddr_t) movieArrow_create},
    {"mapMovieArrowActivate", (caddr_t) mapMovieArrowActivate},
    {"circAvg_list_proc", (caddr_t) circAvg_list_proc},
    {"circAvg_screen_activate", (caddr_t) circAvg_screen_activate},
    {"DrawFFTWindow", (caddr_t) DrawFFTWindow},
    {"DrawCircAvgWindow", (caddr_t) DrawCircAvgWindow},
    {"colormap_ramp_change", (caddr_t) colormap_ramp_change},
    {"movieMinMax_change", (caddr_t) movieMinMax_change},
    {"stepMode_create", (caddr_t) stepMode_create},
    {"stepMode_change", (caddr_t) stepMode_change},
    {"stepMode_button_change", (caddr_t) stepMode_button_change},
    {"label_create", (caddr_t) label_create},
    {"surfDensScroll_create", (caddr_t) surfDensScroll_create},
    {"pifhead_activate", (caddr_t) pifhead_activate},
    {"stat_calc_activate", (caddr_t) stat_calc_activate},
    {"stat_text_create", (caddr_t) stat_text_create},
    {"stats_activate", (caddr_t) stats_activate},
    {"autoBox_activate", (caddr_t) autoBox_activate},  
    {"correlate_activate", (caddr_t) correlate_activate},  
    {"mapShadeScreen_activate", (caddr_t) mapShadeScreen_activate},
    {"subMapShadeScreen_activate", (caddr_t) subMapShadeScreen_activate},
    {"angleText_create", (caddr_t) angleText_create},
    {"surfDensValueChange", (caddr_t) surfDensValueChange},
    {"surfDensScrollChange", (caddr_t) surfDensScrollChange},
    {"surfDensValue_create", (caddr_t) surfDensValue_create},
    {"tryThis_activate", (caddr_t) tryThis_activate},
    {"setFont_create", (caddr_t) setFont_create},
    {"mapShade_activate", (caddr_t) mapShade_activate},
    {"mapMovie_activate", (caddr_t) mapMovie_activate},
    {"mapMovie_show_activate", (caddr_t) mapMovie_show_activate},
    {"editBoxZoomText_changed", (caddr_t) editBoxZoomText_changed},
    {"editBox_save_activate", (caddr_t) editBox_save_activate},
    {"editBox_undo_activate", (caddr_t) editBox_undo_activate},
    {"help_text_create", (caddr_t) help_text_create},
    {"help_save_activate", (caddr_t) help_save_activate},
    {"help_screen_activate", (caddr_t) help_screen_activate},
    {"context_help_screen_activate", (caddr_t) context_help_screen_activate},
    {"rulerText_create", (caddr_t) rulerText_create},
    {"rulerText_changed", (caddr_t) rulerText_changed},
    {"editBoxForm_create", (caddr_t) editBoxForm_create},
    {"editBoxPaintText_changed", (caddr_t) editBoxPaintText_changed},
    {"editBoxText_create", (caddr_t) editBoxText_create},
    {"editBoxPaintArrowActivate", (caddr_t) editBoxPaintArrowActivate},
    {"editBoxZoomArrowActivate", (caddr_t) editBoxZoomArrowActivate},
    {"editBoxArrowActivate", (caddr_t) editBoxArrowActivate},
    {"editBoxText_changed", (caddr_t) editBoxText_changed},
    {"editBoxClick", (caddr_t) editBoxClick},
    {"DrawEditBoxWindow", (caddr_t) DrawEditBoxWindow},
    {"dispMode_change", (caddr_t) dispMode_change},
    {"circAvgMotion", (caddr_t) circAvgMotion},
    {"colormapMotion", (caddr_t) colormapMotion},
    {"colormapValue_create", (caddr_t) colormapValue_create},
    {"DrawTestColormapWindow", (caddr_t) DrawTestColormapWindow},
    {"colorClick", (caddr_t) colorClick},
    {"colorScrollChange", (caddr_t) colorScrollChange},
    {"infoCancel", (caddr_t) infoCancel},
    {"infoOK", (caddr_t) infoOK},
    {"histArrowActivate", (caddr_t) histArrowActivate},
    {"radiusArrowActivate", (caddr_t) radiusArrowActivate},
    {"imgNumArrowActivate", (caddr_t) imgNumArrowActivate},
    {"imgNumActivate", (caddr_t) imgNumActivate},
    {"boxInitActivate", (caddr_t) boxInitActivate},
    {"radiusText_changed", (caddr_t) radiusText_changed},
    {"radiusText_create", (caddr_t) radiusText_create},
    {"imgNumText_changed", (caddr_t) imgNumText_changed},
    {"imgNumText_create", (caddr_t) imgNumText_create},
    {"squareText_changed", (caddr_t) squareText_changed},
    {"featherMode_change", (caddr_t) featherMode_change},
    {"backgroundMode_change", (caddr_t) backgroundMode_change},
    {"boxMode_change", (caddr_t) boxMode_change},
    {"options_activate", (caddr_t) options_activate},
    {"plotScroll_create", (caddr_t) plotScroll_create},
    {"fftClick", (caddr_t) fftClick},
    {"plotClick", (caddr_t) plotClick},
    {"toolMode_change", (caddr_t) toolMode_change},
    {"formResize", (caddr_t) formResize},
    {"plotHorizChange", (caddr_t) plotHorizChange},
    {"plotVertChange", (caddr_t) plotVertChange},
    {"toggle_change", (caddr_t) toggle_change},
    {"tog_create", (caddr_t) tog_create},
    {"lock_create", (caddr_t) lock_create},
    {"drawingWindow_create", (caddr_t) drawingWindow_create},
    {"DrawColormapWindow", (caddr_t) DrawColormapWindow},
    {"dio_change", (caddr_t) dio_change},
    {"colormap_activate", (caddr_t) colormap_activate},
    {"DrawWindowExpose", (caddr_t) DrawWindowExpose},
    {"read_activate", (caddr_t) read_activate},
    {"write_activate", (caddr_t) write_activate},
    {"exit_activate", (caddr_t) exit_activate},
    {"editBox_activate", (caddr_t) editBox_activate},
    {"histogram_activate", (caddr_t) histogram_activate},
    {"plotWindow_create", (caddr_t) plotWindow_create},
    {"generic_deactivate", (caddr_t) generic_deactivate}
};

static int reglist_num = (sizeof reglist / sizeof reglist [0]);

/* Translate keys for TEXT widgets */
char rwa_translation_table[] = 
"<Key>Return : imgNumActivate()";

XtActionsRec rwa_action_table[] = 
    {
      {"imgNumActivate",     (XtActionProc)imgNumActivate},
    };

char fftStep_translation_table[] = 
"<Key>Return : fft_step_changed()";

XtActionsRec fftStep_action_table[] = 
    {
      {"fft_step_changed",     (XtActionProc)fft_step_changed},
    };

char box_translation_table[] = 
"<Key>Return : boxInitActivate()";

XtActionsRec box_action_table[] = 
    {
      {"boxInitActivate",     (XtActionProc)boxInitActivate},
    };

char editBoxText_translation_table[] = 
"<Key>Return : editBoxText_changed()";

XtActionsRec editBoxText_action_table[] = 
    {
      {"editBoxText_changed",     (XtActionProc)editBoxText_changed},
    };

char map_abang_translation_table[] = 
"<Key>Return : crysDisp_cr_activate()";

XtActionsRec map_abang_action_table[] = 
    {
      {"crysDisp_cr_activate",(XtActionProc)crysDisp_cr_activate},
    };

char aoverb_translation_table[] = 
"<Key>Return : crysDisp_cr_activate()";

XtActionsRec aoverb_action_table[] = 
    {
      {"crysDisp_cr_activate",(XtActionProc)crysDisp_cr_activate},
    };

char fftImgNum_translation_table[] = 
"<Key>Return : fftImgNum_cr_activate()";

XtActionsRec fftImgNum_action_table[] = 
    {
      {"fftImgNum_cr_activate",(XtActionProc)fftImgNum_cr_activate},
    };


char ctfCurve_translation_table[] = 
"<Key>Return : ctfCurve_cr_activate()";

XtActionsRec ctfCurve_action_table[] = 
    {
      {"ctfCurve_cr_activate",(XtActionProc)ctfCurve_cr_activate},
    };

char surfDens_translation_table[] = 
"<Key>Return : surfDens_CR()\n\
<BtnDown>Button3 : surfDens_CR()";

XtActionsRec surfDens_action_table[] = 
    {
      {"surfDens_CR",(XtActionProc)surfDens_CR},
    };

char zoom_translation_table[] = 
"<Key>Return : zoomText_change()";

XtActionsRec zoom_action_table[] = 
    {
      {"zoomText_change",(XtActionProc)zoomText_change},
    };

char zoomArrow_translation_table[] = 
"<BtnDown>Button2 : button_zoom_activate()\n\
<BtnDown>Button3 : button_zoom_activate()";

XtActionsRec zoomArrow_action_table[] = 
    {
      {"button_zoom_activate",(XtActionProc)button_zoom_activate},
    };

char imgNum_translation_table[] = 
"<BtnDown>Button2 : imgNumArrowActivateButton()\n\
<BtnDown>Button3 : imgNumArrowActivateButton()";

XtActionsRec imgNum_action_table[] = 
    {
      {"imgNumArrowActivateButton",(XtActionProc)imgNumArrowActivateButton},
    };

char radius_translation_table[] = 
"<BtnDown>Button2 : radiusArrowActivateButton()\n\
<BtnDown>Button3 : radiusArrowActivateButton()";

XtActionsRec radius_action_table[] = 
    {
      {"radiusArrowActivateButton",(XtActionProc)radiusArrowActivateButton},
    };

char boxInit_translation_table[] = 
"<BtnDown>Button1 : boxInitArrowActivateButton()\n\
<BtnDown>Button2 : boxInitArrowActivateButton()\n\
<BtnDown>Button3 : boxInitArrowActivateButton()";

XtActionsRec boxInit_action_table[] = 
    {
      {"boxInitArrowActivateButton",(XtActionProc)boxInitArrowActivateButton},
    };

char boxNum_translation_table[] = 
"<BtnDown>Button2 : boxNumArrowActivateButton()\n\
<BtnDown>Button3 : boxNumArrowActivateButton()";

XtActionsRec boxNum_action_table[] = 
    {
      {"boxNumArrowActivateButton",(XtActionProc)boxNumArrowActivateButton},
    };

char paintSize_translation_table[] = 
"<BtnDown>Button2 : paintSizeArrowActivateButton()\n\
<BtnDown>Button3 : paintSizeArrowActivateButton()";

XtActionsRec paintSize_action_table[] = 
    {
      {"paintSizeArrowActivateButton",
	           (XtActionProc)paintSizeArrowActivateButton},
    };

char icosa_translation_table[] = 
"<BtnDown>Button2 : icosaArrowActivateButton()\n\
<BtnDown>Button3 : icosaArrowActivateButton()";

XtActionsRec icosa_action_table[] = 
    {
      {"icosaArrowActivateButton",
	           (XtActionProc)icosaArrowActivateButton},
    };

char cmScroll_translation_table[] = 
"<BtnDown>Button2 : cmScrollArrowActivateButton()\n\
<BtnDown>Button3 : cmScrollArrowActivateButton()";

XtActionsRec cmScroll_action_table[] = 
    {
      {"cmScrollArrowActivateButton",
	           (XtActionProc)cmScrollArrowActivateButton},
    };

char gbs_translation_table[] = 
"<BtnDown>Button2 : gbsArrowActivateButton()\n\
<BtnDown>Button3 : gbsArrowActivateButton()";

XtActionsRec gbs_action_table[] = 
    {
      {"gbsArrowActivateButton",
	           (XtActionProc)gbsArrowActivateButton},
    };

char ta_translation_table[] = 
"<BtnDown>Button2 : taArrowActivateButton()\n\
<BtnDown>Button3 : taArrowActivateButton()";

XtActionsRec ta_action_table[] = 
    {
      {"taArrowActivateButton",
	           (XtActionProc)taArrowActivateButton},
    };

char ta2_translation_table[] = 
"<BtnDown>Button2 : ta2ArrowActivateButton()\n\
<BtnDown>Button3 : ta2ArrowActivateButton()";

XtActionsRec ta2_action_table[] = 
    {
      {"ta2ArrowActivateButton",
	           (XtActionProc)ta2ArrowActivateButton},
    };

char version_translation_table[] = 
"<Key>Return : version_scroll_activate()";

XtActionsRec version_action_table[] = 
    {
      {"version_scroll_activate",(XtActionProc)version_scroll_activate},
    };

char paramFile_translation_table[] = 
"<Key>Return : paramFile_read_activate()";

XtActionsRec paramFile_action_table[] = 
    {
      {"paramFile_read_activate",(XtActionProc)paramFile_read_activate},
    };

char paramMapFile_translation_table[] = 
"<Key>Return : paramMapFile_read_activate()";

XtActionsRec paramMapFile_action_table[] = 
    {
      {"paramMapFile_read_activate",(XtActionProc)paramMapFile_read_activate},
    };

char paramScanFile_translation_table[] = 
"<Key>Return : paramScanFile_read_activate()";

XtActionsRec paramScanFile_action_table[] = 
    {
      {"paramScanFile_read_activate",(XtActionProc)paramScanFile_read_activate},
    };

char projContour_translation_table[] = 
"<Key>Return : projContour_cr_activate()";

XtActionsRec projContour_action_table[] = 
    {
      {"projContour_cr_activate",(XtActionProc)projContour_cr_activate},
    };

char projasu_translation_table[] = 
"<Key>Return : projasu_cr_activate()";

XtActionsRec projasu_action_table[] = 
    {
      {"projasu_cr_activate",(XtActionProc)projasu_cr_activate},
    };

char ppin_translation_table[] = 
"<BtnDown>Button2 : ppinArrowButton()\n\
<BtnDown>Button3 : ppinArrowButton()";

XtActionsRec ppin_action_table[] = 
    {
      {"ppinArrowButton",(XtActionProc)ppinArrowButton},
    };

char stepPP_translation_table[] = 
"<BtnDown>Button2 : stepPPArrowButton()\n\
<BtnDown>Button3 : stepPPArrowButton()";

XtActionsRec stepPP_action_table[] = 
    {
      {"stepPPArrowButton",(XtActionProc)stepPPArrowButton},
    };

char stepPPtext_translation_table[] = 
"<Key>Return : stepPPtext_changed()";

XtActionsRec stepPPtext_action_table[] = 
    {
      {"stepPPtext_changed",     (XtActionProc)stepPPtext_changed},
    };


char binPP_translation_table[] = 
"<BtnDown>Button2 : binPPArrowButton()\n\
<BtnDown>Button3 : binPPArrowButton()";

XtActionsRec binPP_action_table[] = 
    {
      {"binPPArrowButton",(XtActionProc)binPPArrowButton},
    };

char binPPtext_translation_table[] = 
"<Key>Return : binPPtext_changed()";

XtActionsRec binPPtext_action_table[] = 
    {
      {"binPPtext_changed",     (XtActionProc)binPPtext_changed},
    };


char ppintext_translation_table[] = 
"<Key>Return : ppintext_changed()";

XtActionsRec ppintext_action_table[] = 
    {
      {"ppintext_changed",     (XtActionProc)ppintext_changed},
    };

char strtxpp_translation_table[] = 
"<Key>Return : paramMode_change()";

XtActionsRec strtxpp_action_table[] = 
    {
      {"paramMode_change",     (XtActionProc)paramMode_change},
    };

char ppMinMax_translation_table[] = 
"<Key>Return : ppMinMax_change()";

XtActionsRec ppMinMax_action_table[] = 
    {
      {"ppMinMax_change",     (XtActionProc)ppMinMax_change},
    };

char ctfValue_translation_table[] = 
"<Key>Return : ctfValue_change()";

XtActionsRec ctfValue_action_table[] = 
    {
      {"ctfValue_change",     (XtActionProc)ctfValue_change},
    };

char latText_translation_table[] = 
"<Key>Return : latTextChanged()";

XtActionsRec latText_action_table[] = 
    {
      {"latTextChanged",     (XtActionProc)latTextChanged},
    };


char rotflt_translation_table[] = 
"<BtnDown>Button2 : rotflt_arrow23_activate()\n\
<BtnDown>Button3 : rotflt_arrow23_activate()";

XtActionsRec rotflt_action_table[] = 
    {
      {"rotflt_arrow23_activate",(XtActionProc)rotflt_arrow23_activate},
    };


char cDisp_zArrow_trans_table[] = 
"<BtnDown>Button2 : cDisp_zoom_activate()\n\
<BtnDown>Button3 : cDisp_zoom_activate()";

XtActionsRec cDisp_zArrow_act_table[] = 
    {
      {"cDisp_zoom_activate",(XtActionProc)cDisp_zoom_activate},
    };


char ctfText_translation_table[] = 
"<Key>Return : ctfTextChanged()";

XtActionsRec ctfText_action_table[] = 
    {
      {"ctfTextChanged",     (XtActionProc)ctfTextChanged},
    };

char subText_translation_table[] = 
"<Key>Return : subtractMap_read()";

XtActionsRec subText_action_table[] = 
    {
      {"subtractMap_read",     (XtActionProc)subtractMap_read},
    };

char diffSec_translation_table[] = 
"<Key>Return : diffSec_do()";

XtActionsRec diffSec_action_table[] = 
    {
      {"diffSec_do",     (XtActionProc)diffSec_do},
    };


char phase_translation_table[] = 
"<BtnDown>Button2 : phaseArrowButton()\n\
<BtnDown>Button3 : phaseArrowButton()";

XtActionsRec phase_action_table[] = 
    {
      {"phaseArrowButton",(XtActionProc)phaseArrowButton},
    };


char smMMax_translation_table[] = 
"<BtnDown>Button2 : smMMaxArrowButton()\n\
<BtnDown>Button3 : smMMaxArrowButton()";

XtActionsRec smMMax_action_table[] = 
    {
      {"smMMaxArrowButton",(XtActionProc)smMMaxArrowButton},
    };


char lopText_translation_table[] = 
"<Key>Return : lopTextChanged()";

XtActionsRec lopText_action_table[] = 
    {
      {"lopTextChanged",     (XtActionProc)lopTextChanged},
    };

char hipText_translation_table[] = 
"<Key>Return : hipTextChanged()";

XtActionsRec hipText_action_table[] = 
    {
      {"hipTextChanged",     (XtActionProc)hipTextChanged},
    };


char coravg_translation_table[] = 
"<BtnDown>Button2 : coravgArrowButton()\n\
<BtnDown>Button3 : coravgArrowButton()";

XtActionsRec coravg_action_table[] = 
    {
      {"coravgArrowButton",(XtActionProc)coravgArrowButton},
    };


char lohiRadiusText_trans_table[] = 
"<Key>Return : lohiRadiusChanged()";

XtActionsRec lohiRadiusText_action_table[] = 
    {
      {"lohiRadiusChanged",     (XtActionProc)lohiRadiusChanged},
    };


char fftImgNumArrow_translation_table[] = 
"<BtnDown>Button2 : fftImgNumArrowButton()\n\
<BtnDown>Button3 : fftImgNumArrowButton()";

XtActionsRec fftImgNumArrow_action_table[] = 
    {
      {"fftImgNumArrowButton",(XtActionProc)fftImgNumArrowButton},
    };





/* End of robem.h */





